#include <QtGui/QApplication>
#include <QTextCodec>
#include <iostream>
#include "GUIMainWindow.h"
#include "GUICommon.h"
#include "UIApp.h"

using namespace::std;

void help() {
	QStringList lines;

	lines << "#Usage:";
	lines << "harktool3 [FILE]...";
	lines << "Read the following file: Localization-TF,Separation-TF,Separation-Matrix,TSP-ListFile,ImpulseResponse-ListFile,NoiseLocationFile,MICARY-LocationFile";
	lines << "";
	lines << "When -nogui option enabled:";
	lines << "harktool3 -nogui -mode localization-tf -mic_file FILE -srcinf_file FILE [-fs 16000] [-fft 512] [-ir 16384] [-direct_sound 32] -o FILE";
	lines << "harktool3 -nogui -mode separation-tf -mic_file FILE -srcinf_file FILE [-fs 16000] [-fft 512] [-ir 16384] [-method vector] -o FILE";
	lines << "";
	lines << "#Example:";
	lines << "harktool3 -nogui -mode localization-tf -mic_file ./MICARY-LocationFile -srcinf_file ./TSP-ListFile -o result.dat";
	lines << "harktool3 -nogui -mode separation-tf -mic_file ./MICARY-LocationFile -srcinf_file ./ImpulseResponse-ListFile -o result.tf";
	lines << "";
	lines << "# Argument list only for Sepalation-TF when -nogui enabled";
	lines << "-mode separation-tf   (Transfer Function for Separation)";
	lines << "-method (vector|Matrix) (Normalization Method)";
	lines << "";
	lines << "# Argument list only for Localization-TF when -nogui enabled";
	lines << "-mode localization-tf (Transfer Function for Localization)";
	lines << "-direct_sound:(Direct sound length[sample])";
	lines << "";
	lines << "# Argument list for Sepalation-TF and Localization-TF when -nogui enabled";
	lines << "-mic_file:    (MICARY_LocationFile)";
	lines << "-srcinf_file: (ImpulseResponse-ListFile or TSP-File)";
	lines << "-fs:          (SamplingRate[Hz])";
	lines << "-fft:         (FFT length[sample])";
	lines << "-ir:          (IR Length[sample])";
	lines << "-o:           (Output file)";
	QString text = lines.join("\n");
	text += "\n";
	cout << text.toStdString()	 << endl;

	return;
}

class Option {
	QString args;
public:
	// Localization-TF and Separation-TF
	QString mode;
	QString mic_file;
	QString srcinf_file;
	QString fs;
	QString fft;
	QString ir;
	QString output;
	// Localizaion-TF
	QString direct_sound;
	// Separation-TF
	QString method;
	bool nogui;

	Option(QString args) {
		this->args = args;

		// Localization-TF and Separation-TF
		fs = "16000";
		fft = "512";
		ir = "16384";
		// Only Localizaion-TF
		direct_sound ="32";
		// Only Separation-TF
		method = "vector";
		nogui = false;
	}
	~Option() {}
	void parse() {
		QRegExp reg;

		// -h|--help
		reg.setPattern("(^|.*\\s+)(?:--help|-h)(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(2);

			// 残りの引数を調べる
			QRegExp reg("^\\s*$");
			if(!reg.exactMatch(args)) {
				THROW_WARNING(QString("Inappropriate argument:%1").arg(args));
			}
			help();
			exit(0);
		}
		// --nogui
		reg.setPattern("(^|.*\\s+)(-*-nogui)(\\s+.*|$)"); // GUIの場合
		if (!reg.exactMatch(args)) { // 逆の条件
			nogui = false;
			return; // GUIの場合はここで、Option Parseは終了
		} else {
			args = reg.cap(1) + reg.cap(3);
			nogui = true;
		}

		// ここからCUI
		if (args == "") {
			help(); // CUIでも引数がなかったらヘルプの表示
			exit(0);
		}

		// -mode
		reg.setPattern("(^|.*\\s+)(?:-*-mode\\s+([^\\s]+))(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(3);
			if (reg.cap(2) != "separation-tf" && reg.cap(2) != "localization-tf"	) {
				THROW_WARNING(QString("Inappropriate argument:-mode:%1").arg(reg.cap(2)));
			}
			mode = reg.cap(2);
		} else {
			THROW_WARNING(QString("Unspecified  argument:-mode"));
		}

		// モード別引数
		if (mode == "localization-tf") {
			// -direct_sound
			reg.setPattern("(^|.*\\s+)(?:-*-direct_sound\\s+([\\d]+))(\\s+.*|$)");
			if (reg.exactMatch(args)) {
				args = reg.cap(1) + reg.cap(3);
				direct_sound = reg.cap(2);
			}
		} else if (mode == "separation-tf") {
			// -method
			reg.setPattern("(^|.*\\s+)(?:-*-method\\s+(vector|matrix))(\\s+.*|$)");
			if (reg.exactMatch(args)) {
				args = reg.cap(1) + reg.cap(3);
				method = reg.cap(2);
			}
		}

		// -mic_file
		reg.setPattern("(^|.*\\s+)(?:-*-mic_file\\s+([^\\s]+))(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(3);
			mic_file = reg.cap(2);
		}
		// -srcnf_file
		reg.setPattern("(^|.*\\s+)(?:-*-srcinf_file\\s+([^\\s]+))(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(3);
			srcinf_file = reg.cap(2);
		}

		// -fs
		reg.setPattern("(^|.*\\s+)(?:-*-fs\\s+([\\d]+))(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(3);
			fs = reg.cap(2);
		}
		// -fft
		reg.setPattern("(^|.*\\s+)(?:-*-fft\\s+([\\d]+))(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(3);
			fft = reg.cap(2);
		}
		// -ir
		reg.setPattern("(^|.*\\s+)(?:-*-ir\\s+([\\d]+))(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(3);
			ir = reg.cap(2);
		}
		// -o (Output file)
		reg.setPattern("(^|.*\\s+)(?:-*-o\\s+([^\\s]+))(\\s+.*|$)");
		if (reg.exactMatch(args)) {
			args = reg.cap(1) + reg.cap(3);
			output = reg.cap(2);
		}


		// 残りの引数を調べる
		// 不必要な引数が含まれていた場合は、エラー終了する
		reg.setPattern("^\\s*$");
		if(!reg.exactMatch(args)) {
			THROW_WARNING(QString("Inappropriate argument:%1").arg(args));
		}

		// DEBUG
		IFDEBUG(DP("MODE", mode));
		IFDEBUG(DP("MIC_FILE", mic_file));
		IFDEBUG(DP("SRCINF_FILE", srcinf_file));
		IFDEBUG(DP("FS", fs));
		IFDEBUG(DP("FFT", fft));
		IFDEBUG(DP("IR", ir));
		IFDEBUG(DP("DIRECT_SOUND", direct_sound));
		IFDEBUG(DP("METHOD", method));

	}
};

void gui(Option& option) {
(void)option;
	GUIMainWindow *main_window;
	try {
		// メインウィンドウ
		main_window = GUIMainWindow::getMainWindow();
		main_window->show();
		QStringList list = qApp->arguments();

		if (list.size() == 1) {
			main_window->on_new_file_triggered(); // デフォルトで新規作成のダイアログを表示する
		}
		for (int i = 1; i < list.size(); i++) {
			main_window->openFileByCommandLine(list[i]);
		}
		qApp->exec();
	} catch (...) {
			delete main_window;
			throw;
		}
		delete main_window;
	}

void cui(Option& option) {

	if (option.mode == "localization-tf") {
		// Document 生成
		HT::MUSIC::UIDocument* doc = new HT::MUSIC::UIDocument();
		HT::MUSIC::UIData* data = doc->newTFData(doc);

		HT::MUSIC::CreateTFParam param; // すべてのパラメータを格納
		// 数字パラメータ格納
		param.NFFT = option.fft.toInt();
		param.NFS = option.fs.toInt();
		param.DIRECT_LENGTH = option.direct_sound.toInt();
		param.IMPULSE_RESPONSE_LEN = option.ir.toInt();
		// 入力マイクファイル
		param.mic_file = option.mic_file.toStdString();
		// 入力ソースファイル
		param.src_file = option.srcinf_file.toStdString();

		// データ生成実行
		data->createTFData(param);
		// ファイルに保存する
		data->saveTFFile(option.output.toStdString());

	} else if (option.mode == "separation-tf") {
		// Document 生成
		HT::GSS::UIDocument* doc = new HT::GSS::UIDocument();
		HT::GSS::UIData* data = doc->newTFData(doc);
		HT::GSS::CreateTFParam param; // すべてのパラメータを格納
		// 数字パラメータ格納
		param.NFFT = option.fft.toInt();
		param.NFS = option.fs.toInt();
		param.IMPULSE_RESPONSE_LEN = option.ir.toInt();
		// 入力マイクファイル
		param.mic_file = option.mic_file.toStdString();
		// 入力ソースファイル
		param.src_file = option.srcinf_file.toStdString();

		// 2012-02-22 (s.noguchi) 正規化方法選択
		int normalization;
		if (option.method == "ch") {
			normalization = HT::GSS::UIData::CH;
		} else if (option.method == "ch_frq") {
			normalization = HT::GSS::UIData::CH_FRQ;
		} else if (option.method == "ch_frq_dir") {
			normalization = HT::GSS::UIData::CH_FRQ_DIR;
		} else {
			normalization = HT::GSS::UIData::NONE;
		}
		data->setNormalization(normalization);

		// データ生成実行
		data->createTFData(param);
		// ファイルに保存する
		data->saveTFFile(option.output.toStdString());
	}
}

void start() {

	// コマンド名以外の引数を1行にまとめる
	QStringList list = qApp->arguments();
	list.pop_front();
	QString args = list.join(" ");
	Option option(args);
	option.parse();

	if (option.nogui) {
		cui(option);
	} else {
		gui(option);
	}
}

int main(int argc, char *argv[]) {

	IFDEBUG(DP("Locale=", QLocale::system().name()));

	try {
		QApplication app(argc, argv);

		QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8")); // ソースコード
		QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8")); // 変換コード
//		QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale()); // 変換コード

		// Qtフレームワーク自体の翻訳
		QTranslator qt_translator;
		qt_translator.load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath)); // "/usr/share/qt4/translations/"
		app.installTranslator(&qt_translator);

		// 翻訳ファイルのロード
		QTranslator en_translator;
		QLocale jplocale(QLocale::Japanese);
		if (jplocale.language() != QLocale::system().language()) {
			QLocale locale(QLocale::English);

			QString fname1 = QCoreApplication::applicationDirPath() + "/../share/harktool3/translations/harktool3_" + locale.name();
			QString fname2 = "./translations/harktool3_" + locale.name(); // バイナリと同じディレクトリ
			bool ret = en_translator.load(fname1, qApp->applicationDirPath());
			if (ret) {
				app.installTranslator(&en_translator);
			} else {
				std::cout<< "# Can't open English translation file:" << fname1.toStdString() << std::endl;
			}
		}
		start(); // CUI or GUI started
		return 0;

	} catch (HT::UIException& exp) {
		/// コマンドライン版は、普通にここまで到達する
		if (dynamic_cast<HT::UIWarning*> (&exp)) {
			QString msg = QString("%1:Warning:%2").arg(argv[0]).arg(exp.what());
			cout << msg.toStdString() << endl;
		} else if (dynamic_cast<HT::UICritical*> (&exp)) {
			QString msg = QString("%1:File::%2\nFunction::%3\nLine::%4").arg(argv[0]).arg(exp.getFileName()).arg(
					exp.getFunctionName()).arg(exp.getLineNumber());
			cout << msg.toStdString() << endl;
			msg = QString("%1:CriticalError:%2").arg(argv[0]).arg(exp.what());
			cout << msg.toStdString() << endl;
		///
			IFDEBUG(std::cout << exp.getFileName() << ":" << exp.getFunctionName() << ":" << "Line:" << exp.getLineNumber(););
		} else {
			std::cout<< "# UIException "
				<< exp.getFileName() << ":"
				<< exp.getFunctionName() << ":"
				<< "Line:" << exp.getLineNumber() << ":"
				<< exp.what() << std::endl;
		}
	} catch (std::exception& exp) {
		std::cout << "# std::exception " << exp.what() << std::endl;
	} catch (...) {
		std::cout << "# Unknown user defined exception occurred." << std::endl;
	}
	return 1;
}
