#include "GUICommon.h"
#include "libharkio/libharkio.h"

using namespace std;

void selectCombo(QComboBox *combo, int select) {
	for (int i=0; i < combo->count(); i++) {
		int data = combo->itemData(i).toInt();
		if (data == select) {
			combo->setCurrentIndex(i);
		}
	}
}

void removeComboAll(QComboBox *combo) {
	while (1) {
		int index = combo->currentIndex();
		if (index < 0)
			break;
		combo->removeItem(index);
	}
}


void setWindowTitle(QWidget *widget) {
	widget->setWindowTitle(QApplication::applicationName());
}

void exp_dialog(std::exception& _exp, QWidget* parent) {
	QMessageBox *msgBox = new QMessageBox(parent);
	msgBox->setStandardButtons(QMessageBox::Ok);

	if (dynamic_cast<HT::UIException*>(&_exp)) {
		HT::UIException* exp = dynamic_cast<HT::UIException*>(&_exp);

		if (dynamic_cast<HT::UIWarning*>(exp)) {
			msgBox->setIcon(QMessageBox::Warning);
			msgBox->setText(exp->getMessage());

		} else if (dynamic_cast<HT::UICritical*>(exp)) {
			msgBox->setIcon(QMessageBox::Critical);
			msgBox->setText("Critical Error                                     ");
			msgBox->setInformativeText(exp->getMessage());
			QString msg = QString("File::%1\nFunction::%2\nLine::%3").arg(exp->getFileName()).arg(exp->getFunctionName()).arg(exp->getLineNumber());
			msgBox->setDetailedText(msg);
		}
#ifdef DEBUG
		std::cout << exp->getFileName() << ":" << exp->getFunctionName() << ":" << "Line:" << exp->getLineNumber() << std::endl;
#endif

	} else if (dynamic_cast<harkio::Error*>(&_exp)) {
		harkio::Error* exp = dynamic_cast<harkio::Error*>(&_exp);
		msgBox->setIcon(QMessageBox::Warning);
		msgBox->setText(exp->getMsg().c_str());
#ifdef DEBUG
		std::cout << exp->what() << std::endl;
#endif

	}	else {
		throw _exp;
	}
	msgBox->setModal(true);
	msgBox->show();
}

