#include "GUIImpulseResponseListFileTemplateNew.h"
#include "ui_GUIImpulseResponseListFileTemplateNew.h"

#include <math.h>
#include <QtGui>
#include <QDir>
#include "GUICommon.h"
#include "UIDocument.h"

GUIImpulseResponseListFileTemplateNew::GUIImpulseResponseListFileTemplateNew(
		QWidget *parent) :
	QDialog(parent), ui(new Ui::GUIImpulseResponseListFileTemplateNew) {
	ui->setupUi(this);

	::setWindowTitle(this);
	QDir dir;
	// 初期値の設定

	// インパルス応答リストファイル
	QString path;

#ifdef DEBUG
	path = TEST_FLT_TEMPLATE;
#else
	path = QString("%1/YOUR_PATH/flt_D#DEGREE#_ch*.flt").arg(dir.homePath());
#endif
	ui->file_list_template->setText(path);
}

GUIImpulseResponseListFileTemplateNew::~GUIImpulseResponseListFileTemplateNew() {
	delete ui;
}

void GUIImpulseResponseListFileTemplateNew::fileListTemplate() {

	try {
		QString fname = QFileDialog::getOpenFileName(this, "", HT::UIDocument::getLastPath(ui->file_list_template->text().toStdString()).c_str());
		if (fname != "") {
			ui->file_list_template->setText(fname);
			HT::UIDocument::setLastPath(fname.toStdString());
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIImpulseResponseListFileTemplateNew::createTemplate() {

	try {
		QStringList lines;

		double from = ui->phi_from->value();
		double to = ui->phi_to->value();
		from = (from >= to ? from - 360 : from);
		double interval = ui->interval->value();
		double radius = ui->radius->value();
		double theta = ui->theta->value();
		QString list_file_template = ui->file_list_template->text();
		int offset = ui->offset->value();

		double count = double(to - from) / interval;
		if (count == (to - from) / interval) {
			count++;
		}
		lines << QString("# ImpulseResponse-ListFile generated from harktool by %1.").arg(getUserName());
		lines << QString("# DateTime: %1").arg(QDateTime::currentDateTime().toString("yyyy/MM/dd HH:mm:ss"));
		lines << "ImpulseResponse-ListFile";
		lines << "Version = 1.0";
		lines << "";
//		lines << "# 音源座標とインパルス応答ファイルのリスト";
//		lines << "# x, y, z %% インパルス応答のフルパスファイル名, 読み取り開始フレーム数";
		lines << "# list of sound source position and TSP file";
		lines << "# x, y, z %% TSP file name, IR start position in sample(Starting with 1)";


		for (double phi = from, i = 0; phi <= to; phi += interval, i++) { // 小数のループに注意
			double x = radius * cos(theta * M_PI / 180.0) * cos(phi * M_PI / 180.0);
			double y = radius * cos(theta * M_PI / 180.0) * sin(phi * M_PI / 180.0);
			double z = radius * sin(theta * M_PI / 180.0);

			QString filename(list_file_template);
			QString angle = QString("%1").arg((phi < 0? phi + 360: phi), 3+1+2, 'f', 2, QLatin1Char('0')); // "045.05"の様な文字列の作成
			angle.replace(".", "_");
			filename.replace("#DEGREE#", QString("%1").arg(angle));
			lines << QString("%1, %2, %3 \%\%%4, %5").arg(x,7,'f',4).arg(y,7,'f',4).arg(z,7,'f',4).arg(filename).arg(offset);
		}
		// 最終行にはノイズの音源の情報を付加する
//		lines << "# 次行の音源方向は分離用伝達関数の作成時にのみ使用されます";
		lines << "# The following line is used for TF generation for separation only when FIXED_MODE=true";
		lines << QString("#%1, %2, %3 \%\%%4, %5").arg(0.0,7,'f',4).arg(0.0,7,'f',4).arg(-1.0,7,'f',4).arg("IMPULSE_RESPONSE_FOR_NOISE").arg(offset);

		QString text = lines.join("\n");
		text += "\n";
		emit createdTemplate(text);

		close();
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUIImpulseResponseListFileTemplateNew::changeEvent(QEvent *e) {
	QDialog::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
