#include "GUILocalizationTFNew.h"
#include "ui_GUILocalizationTFNew.h"

#include "UIDatData.h"
#include "GUICommon.h"
#include "GUILocalizationTF.h"
#include "GUIImpulseResponseListFile.h"
#include "GUITSPListFile.h"
#include "GUIMicaryLocationFile.h"

#include "UIApp.h"
#include "UIGraph.h"
#include "UIDocument.h"
#include "libharkio/libharkio.h"

using namespace harkio;

GUILocalizationTFNew::GUILocalizationTFNew(QWidget *parent) :
	QDialog(parent), ui(new Ui::GUILocalizationTFNew) {
	ui->setupUi(this);

	::setWindowTitle(this);

	QString srcinf;
	QString micfile;
	QDir dir;
#ifdef DEBUG
	micfile = TEST_MICFILE;
	srcinf = TEST_SRCINF;
#else
	micfile = QString("%1/YOUR_PATH/MICARY-LocationFile").arg(dir.homePath());
	srcinf = QString("%1/YOUR_PATH/ImpulseResponse-ListFile or TSP-ListFile").arg(dir.homePath());
#endif

	connect(ui->mic_file, SIGNAL(activated(int)), this, SLOT(changeComboBox(int)));
	connect(ui->srcinf_file, SIGNAL(activated(int)), this, SLOT(changeComboBox(int)));
	ui->mic_file->addItem(micfile);
	ui->srcinf_file->addItem(srcinf);

	ui->experimental->setChecked(true);
	ui->impulseresponse->setChecked(true);

	making_method_clicked();
	signal_data_clicked();
}

void GUILocalizationTFNew::changeComboBox(int index) {

	try {
		QComboBox* combo = qobject_cast<QComboBox*>(sender());
		if (index) {
			combo->lineEdit()->setReadOnly(true);
		} else {
			combo->lineEdit()->setReadOnly(false);
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUILocalizationTFNew::updateComboBox() {

	for (int i=0; i < tsp_files.size(); i++) {
		ui->srcinf_file->addItem(tsp_files[i].first, tsp_files[i].second);
	}
	for (int i=0; i < impulse_files.size(); i++) {
		ui->srcinf_file->addItem(impulse_files[i].first, impulse_files[i].second);
	}
	for (int i=0; i < mic_files.size(); i++) {
		ui->mic_file->addItem(mic_files[i].first, mic_files[i].second);
	}
}

void GUILocalizationTFNew::mic_location_clicked() {

	try {
		QString fname = QFileDialog::getOpenFileName(this, "MICARY-LocationFile", HT::UIDocument::getLastPath(ui->mic_file->lineEdit()->text().toStdString()).c_str());
		if (fname != "") {
			ui->mic_file->lineEdit()->setText(fname);
			HT::UIDocument::setLastPath(fname.toStdString());
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUILocalizationTFNew::src_location_clicked() {

	try {
		QString title;
		if (ui->impulseresponse->isChecked()) {
			title = "ImpulseResponse-ListFile";
		} else {
			title = "TSP-ListFile";
		}
		QString fname = QFileDialog::getOpenFileName(this, title, HT::UIDocument::getLastPath(ui->srcinf_file->lineEdit()->text().toStdString()).c_str());

		if (fname != "") {
			ui->srcinf_file->lineEdit()->setText(fname);
			HT::UIDocument::setLastPath(fname.toStdString());
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUILocalizationTFNew::making_method_clicked() {

	try {
		if (ui->experimental->isChecked()) {
			ui->degree_intervals->setEnabled(false);
			ui->music_dir->setEnabled(true);
			ui->impulseresponse->setEnabled(true);
			ui->tsp->setEnabled(true);
			ui->srcinf_file->setEnabled(true);
			ui->impulse_response_length->setEnabled(true);
		} else if (ui->theoretical->isChecked()) {
			ui->degree_intervals->setEnabled(true);
			ui->music_dir->setEnabled(false);
			ui->impulseresponse->setEnabled(false);
			ui->tsp->setEnabled(false);
			ui->srcinf_file->setEnabled(false);
			ui->impulse_response_length->setEnabled(false);
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}
void GUILocalizationTFNew::signal_data_clicked() {

	try {
		if (ui->impulseresponse->isChecked()) {
			ui->srcinf_label->setText("ImpulseResponse-ListFile");
		} else if (ui->tsp->isChecked()) {
			ui->srcinf_label->setText("TSP-ListFile");
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUILocalizationTFNew::theoretical_create() {

	GUILocalizationTF* guidoc = NULL;
	HT::MUSIC::UIDocument *doc = NULL;
	HT::MUSIC::UIData *data = NULL;

	HT::MUSIC::MKMusicParam param;
	std::string fname;

	try {
		{ // エラーチェック

			// MICARY-LocationFile のチェック
			QString imp_fname = ui->mic_file->itemData(ui->mic_file->currentIndex()).toString();
			if (imp_fname != "") {
				fname = imp_fname.toStdString();
			} else {
				fname = ui->mic_file->lineEdit()->text().toStdString();
			}
			if (fname == "") {
				THROW_WARNING(tr("Input MICARY-LcationFile", "MICARY-LcationFileを入力してください"));
			}
			harkio::TextData text;
			int type = text.checkFileIdentifier(fname);
			HT::UIApp *app = HT::UIApp::getInstance();
			if (app->convIDHarkToolTextToDocument(type) != HT::UIApp::MIC) {
				THROW_WARNING(tr("Unknown format:MICARY-LcationFile", "MICARY-LcationFileが不明なフォーマットです"));
			}
		}
		ui->degree_intervals->setEnabled(true);
		ui->music_dir->setEnabled(false);
		doc = new HT::MUSIC::UIDocument();
		data = doc->newTFData(doc);

		param.mic_file = fname;
		param.NFS = ui->fs->value();
		param.NFFT =ui->fft->value();
		param.DEGREE_INTERVAL = ui->degree_intervals->value();
		param.IMPULSE_RESPONSE_LEN = ui->impulse_response_length->value();
		// MKMUSICTFの実行
		doc->mkmusictf(param);
		guidoc = new GUILocalizationTF(doc);

	} catch (...) {
		throw;
	}
	emit createdData(guidoc);
	return;

}

void GUILocalizationTFNew::experimental_create() {

	GUILocalizationTF* guidoc = NULL;
	HT::MUSIC::UIDocument *doc = NULL;
	HT::MUSIC::UIData *data = NULL;

	try {
		HT::MUSIC::CreateTFParam param; // すべてのパラメータを格納

		// 数字パラメータ格納
		param.NFFT = ui->fft->value();
		param.NFS = ui->fs->value();
		param.DIRECT_LENGTH = ui->music_dir->value();
		param.IMPULSE_RESPONSE_LEN = ui->impulse_response_length->value();

		HT::UIApp *app = HT::UIApp::getInstance();

		{ // エラーチェック
			std::string fname;
			QString imp_fname;
			harkio::TextData text;
			int type;

			// MICARY-LocationFile のチェック
			imp_fname = ui->mic_file->itemData(ui->mic_file->currentIndex()).toString();
			if (imp_fname != "") {
				fname = imp_fname.toStdString();
			} else {
				fname = fname = ui->mic_file->lineEdit()->text().toStdString();
			}
			if (fname == "") {
				THROW_CRITICAL(tr("Input MICARY-LcationFile", "MICARY-LcationFileを入力してください"));
			}
			type = text.checkFileIdentifier(fname);
			if (app->convIDHarkToolTextToDocument(type) != HT::UIApp::MIC) {
				THROW_WARNING(tr("Unknown Format:MICARY-LcationFile", "MICARY-LcationFileが不明なフォーマットです"));
			}
			param.mic_file = fname; // 入力マイクファイル

			// ImpulseResponse-ListFIle, TSP-ListFile のチェック
			imp_fname = ui->srcinf_file->itemData(ui->srcinf_file->currentIndex()).toString();
			if (imp_fname != "") {
				fname = imp_fname.toStdString();
			} else {
				fname = ui->srcinf_file->lineEdit()->text().toStdString();
			}
			if (fname == "") {
				if (ui->impulseresponse->isChecked()) {
					THROW_WARNING(tr("Input ImpulseResponse-ListFile", "ImpulseResponse-ListFileを入力してください"));
				} else {
					THROW_WARNING(tr("Input TSP-ListFile", "TSP-ListFileを入力してください"));
				}
			}
			param.src_file = fname; // 入力ソースファイル

			type = text.checkFileIdentifier(fname);
			if (ui->impulseresponse->isChecked()) {
				if (app->convIDHarkToolTextToDocument(type) != HT::UIApp::IMPULSE) {
					THROW_WARNING(tr("Unknown format:ImpulseResponse-ListFile", "ImpulseResponse-ListFileが不明なフォーマットです"));
				}
			} else if (ui->tsp->isChecked()) {
				if (app->convIDHarkToolTextToDocument(type) != HT::UIApp::TSP) {
					THROW_WARNING(tr("Unknown format:TSP-ListFile", "TSP-ListFileが不明なフォーマットです"));
				}
			}
		} //エラーチェック
		doc = new HT::MUSIC::UIDocument();
		data = doc->newTFData(doc);

		// ダイアログ更新イベントの取得
		connect(data, SIGNAL(createdTSP(int,int,HT::UIData*)), this, SLOT(createdTSP(int,int,HT::UIData*)));
		// データ作成 ここが時間がかかる
		data->createTFData(param);
		connect(data, SIGNAL(createdTSP(int,int,HT::UIData*)), this, SLOT(createdTSP(int,int,HT::UIData*)));

		// SPECデータの上書き
		harkio::HARK *hark = data->getHARKIO();
		SPEC* spec = hark->getData()->getChunk<harkio::SPEC> ();
		spec->setDate(QDateTime::currentDateTime().toString("yyyy/MM/dd").toStdString().c_str());
		spec->setCreator(getUserName().toStdString().c_str());

		guidoc = new GUILocalizationTF(doc);

	} catch (...) {
		if (guidoc) {
			delete guidoc;
		} else if(doc) {
			delete doc;
		} else if (data) {
			delete data;
		}
		throw;
	}
	// ウィンドウに通知
	emit createdData(guidoc);
}

void GUILocalizationTFNew::ok_clicked() {

	try {
		if (ui->theoretical->isChecked()) {
			theoretical_create();
		} else {
			experimental_create();
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	done(-1);
}

void GUILocalizationTFNew::createdTSP(int current, int total, HT::UIData *data) {

	static QProgressDialog *progress=NULL;
	if (current < 0) {
		delete progress;
		progress = NULL;
		return;
	}
	if (progress == NULL) {
		// プログレスバーダイアログの作成
		progress = new QProgressDialog(this);
		progress->setWindowTitle(tr("Generating impulse response ...", "インパルス応答の作成中..."));
		progress->setRange(0, total);
		progress->setModal(true);
	}
	if (progress->wasCanceled()) {
		delete progress;
		progress = NULL;
		// シグナルの送信元にキャンセルフラグを立てる
		safeCast<HT::MUSIC::UIData*>(data)->setTSPCanceled(true);
		return;
	}
	progress->setValue(current);
}

GUILocalizationTFNew::~GUILocalizationTFNew() {
IFDEBUG(DP("GUILocalizationTFNew destructor"));
	delete ui;
}

void GUILocalizationTFNew::changeEvent(QEvent *e) {
	QDialog::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
