#include "GUINoiseLocationFile.h"
#include "ui_GUINoiseLocationFile.h"

#include "GUICommon.h"
#include "GUINoiseLocationFileTemplateNew.h"
#include "UIDatDocument.h"
#include "libharkio/libharkio.h"
#include "UIGraph.h"
#include "UIData.h"

using namespace harkio;

GUINoiseLocationFile::GUINoiseLocationFile(HT::NOISE::UIDocument* doc, QWidget *parent) :
	QWidget(parent), ui(new Ui::GUINoiseLocationFile) {
	ui->setupUi(this);

	if (doc) {
		this->doc = doc;
	} else {
		this->doc = new HT::NOISE::UIDocument();
	}
}

GUINoiseLocationFile::~GUINoiseLocationFile() {
	delete ui;
	delete doc;
}

void GUINoiseLocationFile::templateNew() {

	try {
		GUINoiseLocationFileTemplateNew dialog(this);

		connect(&dialog, SIGNAL(createdTemplate(QString)), this, SLOT(dispTemplate(QString)));
		dialog.exec();
		disconnect(&dialog, SIGNAL(createdTemplate(QString)), this, SLOT(dispTemplate(QString)));
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUINoiseLocationFile::openFile(QString file) {

	std::ifstream ifs(file.toStdString().c_str());
	if(ifs.fail())
	{
		THROW_WARNING(QString(tr("Can't open file:%1")).arg(file));
	}
	std::stringstream buf;
	buf << ifs.rdbuf();
	dispTemplate(buf.str().c_str());

	doc->setFileName(file.toStdString());
}


void GUINoiseLocationFile::dispTemplate(QString text) {

	try {
		ui->text->setPlainText(text);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUINoiseLocationFile::dispGraph() {

	try {

		QString text = ui->text->toPlainText();
		// ファイルの取得
		harkio::NoiseLocationFile file;
		file.readText(text.toStdString());

		// データの取得
		std::vector<harkio::UINoise<> > *noise_lists = &(file.noise_lists);
		std::vector<double> xyz(3);
		std::vector<std::vector<double> > lists; // マイク位置

		for (uint i=0 ; i < noise_lists->size(); i++) {
			xyz[0] = (*noise_lists)[i].x;
			xyz[1] = (*noise_lists)[i].y;
			xyz[2] = (*noise_lists)[i].z;
			lists.push_back(xyz);
		}
		// プロット
		ScatterPlot<double> graph(&lists);
		graph.setTitle("Noise-LocationFile");
		graph.setStyle(ScatterPlot<>::PlotStyle::LINESPOINTS);
		graph.plot(harktoolDir());

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUINoiseLocationFile::saveFile(bool dialog) {
	try {

		QString fname = doc->getFileName().c_str();
		bool saved=false;
		if (!fname.isEmpty()) {
			saved = true;
		}
		if (dialog || !saved) {
			fname = QFileDialog::getSaveFileName(this, "", doc->getLastPath(fname.toStdString()).c_str());
			if (fname.isEmpty()) {
				return;
			}
		}
		
		QFile file(fname);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
			THROW_WARNING(QString(tr("Can't open file:%1")).arg(fname));
			return;
		}
		doc->setFileName(fname.toStdString());
		HT::UIDocument::setLastPath(fname.toStdString());
		
		// 書き込み
		QTextStream out(&file);
		out << ui->text->toPlainText();
		if (!dialog && saved) {
			DISP_INFO(tr("File Overwritten", "上書き保存しました"));
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}

}

void GUINoiseLocationFile::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
