#include "GUIOldFormatSelect.h"
#include "ui_GUIOldFormatSelect.h"

#include "GUICommon.h"
#include "GUIMainWindow.h"
#include "GUILocalizationTF.h"
#include "GUISeparationTF.h"
#include "GUISeparationMatrix.h"

GUIOldFormatSelect::GUIOldFormatSelect(QWidget *parent) :
	QDialog(parent), ui(new Ui::GUIOldFormatSelect) {
	ui->setupUi(this);

	::setWindowTitle(this);

	removeComboAll(ui->type_combo);

	ui->type_combo->addItem(QString("%1").arg("TF_CONJ"), QVariant(TF_CONJ));
	ui->type_combo->addItem(QString("%1").arg("SM"), QVariant(SM));

	uiUpdate();
}

GUIOldFormatSelect::~GUIOldFormatSelect() {
	delete ui;
}

void GUIOldFormatSelect::openFile() {

	GUIMainWindow *main = GUIMainWindow::getMainWindow();
	GUITabDocument* guidoc = NULL;

	try {

		if (ui->localization->isChecked()) { // Localization TF
			HT::MUSIC::UIDocument *doc = NULL;
			HT::MUSIC::UIData *data = NULL;
			doc = new HT::MUSIC::UIDocument();

//			data = doc->newTFData(doc);
			doc->loadFileNormally(fname.toStdString()); // 旧ファイル読み込み

			data = doc->getUIData(doc);
			int fs = ui->fs->value();
			data->setNFS(fs); // 旧ファイルにはない情報をセット
			guidoc = new GUILocalizationTF(doc);

		} else if (ui->separation->isChecked()) { // 分離行列の伝達関数or分離行列
			HT::GSS::UIDocument* doc = NULL;
			HT::GSS::UIData* data = NULL;
			doc = new HT::GSS::UIDocument();

//			data = doc->newTFData(doc);
			doc->loadFileNormally(fname.toStdString()); // 旧ファイル読み込み
			data = doc->getUIData(doc);

			int type = ui->type_combo->itemData(ui->type_combo->currentIndex()).toInt();
			if (type == SM) { // Separation Matrix
				guidoc = new GUISeparationMatrix(doc);
				data->setFileType(HT::GSS::UIData::SM);
			} else if (type == TF_CONJ) { // Separation TF
				data->setFileType(HT::GSS::UIData::TF_CONJ);
				guidoc = new GUISeparationTF(doc);
			} else {
				THROW_CRITICAL("Unknown file type");
			}
		} else {
			THROW_CRITICAL("Unknown convertion target");
		}
	} catch (std::exception& exp) {
		delete guidoc;
		DISP_DIALOG(exp);
		return;
	}
	main->createDocument(guidoc); // slot
	done(0);
}

void GUIOldFormatSelect::uiUpdate() {

	if (ui->localization->isChecked()) {
		ui->type_combo->setEnabled(false);
		ui->fs->setEnabled(true);
	} else {
		ui->type_combo->setEnabled(true);
		ui->fs->setEnabled(false);
	}
}

void GUIOldFormatSelect::changeEvent(QEvent *e) {
	QDialog::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
