#include "GUIPlotParameterPanel.h"
#include "ui_GUIPlotParameterPanel.h"

#include <QtGui>
#include <typeinfo>
#include <complex>

#include "GUICommon.h"
#include "UIGraph.h"
#include "UIDatData.h"

GUIPlotParameterPanel::GUIPlotParameterPanel(HT::UIDocument* doc, QWidget *parent) :
	QDialog(parent), ui(new Ui::GUIPlotParameterPanel) {

	ui->setupUi(this);
	::setWindowTitle(this);

	this->doc = doc;

	HT::Transform transform;
	transform.BasicData(*doc->getBaseUIData(), &mic, &src, &window, &fs);

	// プロットスタイルのコンボボックスデフォルト
	setProtStyleCombo();
	selectCombo(ui->plot_style, GNUPlot<>::PlotStyle::LINES);
	selectCombo(ui->surface, GNUPlot<>::Surface::LINE);
	selectCombo(ui->color_map, GNUPlot<>::ColorMap::UNDEFINE);
	connectPlotStyle(true); // ここから、依存しているコンボボックスが動的に作成されます

	// 表示対象のコンボボックスのデフォルト
	setDisplayTargetToCombo(ui->display_target);
	ui->graph->setDisabled(true);
	ui->other_value_combo->view()->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
	ui->other_value_combo2->view()->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
	ui->display_target->setCurrentIndex(-1); // コンボボックスのchangeイベントを必ず発生させる細工
	connectPlotCombo(true); // ここから、依存しているコンボボックスが動的に作成されます
	selectCombo(ui->display_target, GUIPlotParameterPanel::TARGET::TF);

	// 表示範囲のテキストボックス
	QRegExpValidator *valid = new QRegExpValidator(QRegExp("[\\-+]?[0-9]+\\.?[0-9]*(?:[eE][\\-+]?[0-9]+|)"), this);
	ui->x_from->setValidator(valid);
	ui->x_to->setValidator(valid);
	ui->y_from->setValidator(valid);
	ui->y_to->setValidator(valid);
	ui->z_from->setValidator(valid);
	ui->z_to->setValidator(valid);
}

void GUIPlotParameterPanel::connectPlotStyle(bool value) {

	if (value) {
		connect(ui->plot_style, SIGNAL(currentIndexChanged(int)), this, SLOT(plotStyleInteract()));
		connect(ui->surface, SIGNAL(currentIndexChanged(int)), this, SLOT(plotStyleInteract()));
		connect(ui->color_map, SIGNAL(currentIndexChanged(int)), this, SLOT(plotStyleInteract()));
	} else {
		disconnect(ui->plot_style, SIGNAL(currentIndexChanged(int)), this, SLOT(plotStyleInteract()));
		disconnect(ui->surface, SIGNAL(currentIndexChanged(int)), this, SLOT(plotStyleInteract()));
		disconnect(ui->color_map, SIGNAL(currentIndexChanged(int)), this, SLOT(plotStyleInteract()));
	}
}

void GUIPlotParameterPanel::connectPlotCombo(bool value) {

	if (value) {
		connect(ui->display_target, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		connect(ui->x_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		connect(ui->y_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		connect(ui->other_value_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		connect(ui->other_value_combo2, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
	} else {
		disconnect(ui->display_target, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		disconnect(ui->x_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		disconnect(ui->y_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		disconnect(ui->other_value_combo, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
		disconnect(ui->other_value_combo2, SIGNAL(currentIndexChanged(int)), this, SLOT(parameterInteract()));
	}
}

int GUIPlotParameterPanel::otherUnit(int target, int x_unit, int y_unit) {

	if (x_unit == y_unit) {
		return GUIPlotParameterPanel::TF_AXIS::UNDEFINE;
	}

	if (target == GUIPlotParameterPanel::TARGET::TF || target == GUIPlotParameterPanel::TARGET::DIFF) {
		int total = GUIPlotParameterPanel::TF_AXIS::FREQ +
		GUIPlotParameterPanel::TF_AXIS::SRCPOS +
		GUIPlotParameterPanel::TF_AXIS::MIC;
		return total - (x_unit + y_unit);
	} else if (target == GUIPlotParameterPanel::TARGET::IFFT) {
		int total = GUIPlotParameterPanel::TF_AXIS::TIME +
		GUIPlotParameterPanel::TF_AXIS::SRCPOS +
		GUIPlotParameterPanel::TF_AXIS::MIC;
		return total - (x_unit + y_unit);
	} else {
		THROW_CRITICAL(QString("Unknown unit id:%1").arg(target));
	}
}

void GUIPlotParameterPanel::setMicCombo(QComboBox *combo) {

	removeComboAll(combo);
	for (int i = 0; i < mic; i++) {
		combo->addItem(QString("%1").arg(i), QVariant(i));
	}

}
void GUIPlotParameterPanel::setFreqCombo(QComboBox *combo) {

	removeComboAll(combo);
	for (int i = 0; i < window / 2; i++) {
		combo->addItem(QString("%1(%2)").arg(i * fs / window).arg(i), QVariant(i));
	}

}

void GUIPlotParameterPanel::setTimeCombo(QComboBox *combo) {

	removeComboAll(combo);
	for (int i = 0; i < window / 2; i++) {
		combo->addItem(QString("%1(%2)").arg(double(i)/ fs, 7,'f',5).arg(i), QVariant(i));
	}

}

void GUIPlotParameterPanel::setSrcPosCombo(QComboBox *combo) {

	removeComboAll(combo);
	for (int i = 0; i < src; i++) {
		combo->addItem(QString("%1").arg(i), QVariant(i));
	}
}

void GUIPlotParameterPanel::plotStyleInteract() {

	try {
		QObject *obj = sender();

		if (obj == ui->plot_style) {

			connectPlotStyle(false);

			int id = ui->plot_style->itemData(ui->plot_style->currentIndex()).toInt();
			if (id == GNUPlot<>::PlotStyle::DOTS || id == GNUPlot<>::PlotStyle::POINTS) {
				int id = ui->color_map->itemData(ui->color_map->currentIndex()).toInt();
				if (id == GNUPlot<>::ColorMap::UNDEFINE) {
					ui->surface->setDisabled(true);
					selectCombo(ui->surface, GNUPlot<>::Surface::UNDEFINE);
				}
			} else {
				ui->surface->setDisabled(false);
			}
			connectPlotStyle(true);

		} else if (obj == ui->surface) {
			connectPlotStyle(false);
			int id = ui->surface->itemData(ui->surface->currentIndex()).toInt();
			if (id == GNUPlot<>::Surface::MESH) {
				ui->color_map->setDisabled(false);
			} else if (id == GNUPlot<>::Surface::LINE) {
				selectCombo(ui->color_map, GNUPlot<>::ColorMap::UNDEFINE);
				ui->color_map->setDisabled(true);
			} else {
				ui->color_map->setDisabled(false);
			}
			connectPlotStyle(true);
		} else if (obj == ui->color_map) {
			connectPlotStyle(false);
			int id = ui->color_map->itemData(ui->color_map->currentIndex()).toInt();

			if (id == GNUPlot<>::ColorMap::PM3D_MAP) {
				selectCombo(ui->plot_style, GNUPlot<>::PlotStyle::UNDEFINE);
			}

			if (id == GNUPlot<>::ColorMap::UNDEFINE) {
				ui->surface->setDisabled(false);
				selectCombo(ui->surface, GNUPlot<>::Surface::UNDEFINE);
			} else {
				ui->surface->setDisabled(true);
				selectCombo(ui->surface, GNUPlot<>::Surface::MESH);
			}
			connectPlotStyle(true);
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIPlotParameterPanel::parameterInteract() {

	try {
		QObject *obj = sender();

		if (obj == ui->other_value_combo || obj == ui->other_value_combo2) {
			return;
		}

		int index = ui->display_target->currentIndex();
		int target = ui->display_target->itemData(index).toInt();

		// target に応じたデフォルトのコンボボックスを作成します
		if (obj == ui->display_target) {

			connectPlotCombo(false);
			// まず初めに、コンボボックスやボタンを無効にしておきます
			removeComboAll(ui->other_value_combo);
			removeComboAll(ui->other_value_combo2);
			ui->other_value_string->setText(tr("Undefined"));
			ui->other_value_string2->setText(tr("Undefined"));
			ui->other_value_string2->setDisabled(true);
			// z軸単位
			ui->z_unit_string->setDisabled(true);
			ui->z_unit->setDisabled(true);
			// エクスポートボタン
			ui->export_data->setDisabled(true);

			switch (target) {
			case GUIPlotParameterPanel::TARGET::TF:
				setTFAxisToCombo(ui->x_combo);
				setTFAxisToCombo(ui->y_combo);
				selectCombo(ui->x_combo, GUIPlotParameterPanel::TF_AXIS::FREQ);
				selectCombo(ui->y_combo, GUIPlotParameterPanel::TF_AXIS::SRCPOS);
				// z軸単位
				ui->z_unit_string->setDisabled(false);
				ui->z_unit->setDisabled(false);
				break;
			case GUIPlotParameterPanel::TARGET::IFFT:
				setIFFTAxisToCombo(ui->x_combo);
				setIFFTAxisToCombo(ui->y_combo);
				selectCombo(ui->x_combo, GUIPlotParameterPanel::TF_AXIS::TIME);
				selectCombo(ui->y_combo, GUIPlotParameterPanel::TF_AXIS::SRCPOS);

				break;
			case GUIPlotParameterPanel::TARGET::DIFF:
				setTFAxisToCombo(ui->x_combo);
				setTFAxisToCombo(ui->y_combo);
				selectCombo(ui->x_combo, GUIPlotParameterPanel::TF_AXIS::FREQ);
				selectCombo(ui->y_combo, GUIPlotParameterPanel::TF_AXIS::SRCPOS);
				// z軸単位
				ui->z_unit_string->setDisabled(false);
				ui->z_unit->setDisabled(false);
				break;
			default:
				THROW_CRITICAL(QString("Unknown target id:%1").arg(target));
				connectPlotCombo(true);
				break;
			}
			connectPlotCombo(true);

		}

		// それ以外のコンボボックスを動的に作成します
		if (obj == ui->display_target || obj == ui->x_combo || obj == ui->y_combo) {

			// 切断
			connectPlotCombo(false);

			int x = ui->x_combo->itemData(ui->x_combo->currentIndex()).toInt();
			int y = ui->y_combo->itemData(ui->y_combo->currentIndex()).toInt();
			int other = otherUnit(target, x, y);

			if (x != y && x != other && y != other) {

				ui->graph->setDisabled(false);

				int id = ui->display_target->itemData(ui->display_target->currentIndex()).toInt();
				bool diff=false;
				if (id == GUIPlotParameterPanel::TARGET::DIFF) {
					diff=true;
					ui->other_value_string2->setDisabled(false);
				} else {
					ui->export_data->setDisabled(false);
				}

				if (other == GUIPlotParameterPanel::TF_AXIS::MIC) {
					ui->other_value_string->setText(tr("Microphone index for display"));
					ui->other_value_string2->setText(tr("Microphone index for calculating difference"));

					setMicCombo(ui->other_value_combo);
					if (diff) {
						setMicCombo(ui->other_value_combo2);
					}
				} else if (other == GUIPlotParameterPanel::TF_AXIS::FREQ) {
					ui->other_value_string->setText(tr("Frequency index for display"));
					ui->other_value_string2->setText(tr("Frequency index for calculating difference"));

					setFreqCombo(ui->other_value_combo);
					if (diff) {
						setFreqCombo(ui->other_value_combo2);
					}
				} else if (other == GUIPlotParameterPanel::TF_AXIS::TIME) {
					ui->other_value_string->setText(tr("Time index for display"));
					ui->other_value_string2->setText(tr("Time index for calculating difference"));

					setTimeCombo(ui->other_value_combo);
					if (diff) {
						setTimeCombo(ui->other_value_combo2);
					}
				} else if (other == GUIPlotParameterPanel::TF_AXIS::SRCPOS) {
					ui->other_value_string->setText(tr("Source index for display"));
					ui->other_value_string2->setText(tr("Source index for calculating difference"));

					setSrcPosCombo(ui->other_value_combo);
					if (diff)
						setSrcPosCombo(ui->other_value_combo2);
				}

			} else {
				removeComboAll(ui->other_value_combo);
				removeComboAll(ui->other_value_combo2);
				ui->graph->setDisabled(true);
				ui->export_data->setDisabled(true);
				ui->other_value_string->setText("Undefined");
				ui->other_value_string2->setText("Undefined");
			}
			// 接続
			connectPlotCombo(true);

		} else {
			THROW_CRITICAL(QString("Unknown ojectct send:%1").arg(obj->objectName()));
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}

}


void GUIPlotParameterPanel::presetDiffGraph(GNUPlot<std::complex<double>, 3>& graph) {

	// 軸の定義
	std::vector<int> indexes(3);
	int x_axis, y_axis, other_axis, other_axis_index1, other_axis_index2;

	x_axis
			= ui->x_combo->itemData(ui->x_combo->currentIndex()).toInt();
	y_axis
			= ui->y_combo->itemData(ui->y_combo->currentIndex()).toInt();

	other_axis = otherUnit(GUIPlotParameterPanel::TARGET::DIFF, x_axis,y_axis);
	other_axis_index1 = ui->other_value_combo->itemData(
			ui->other_value_combo->currentIndex()).toInt();
	other_axis_index2 = ui->other_value_combo2->itemData(
			ui->other_value_combo2->currentIndex()).toInt();

	setIndex(&indexes, GNUPlot<>::AXIS::X, x_axis);
	setIndex(&indexes, GNUPlot<>::AXIS::Y, y_axis);
	setIndex(&indexes, 0, other_axis);

	// 差分の対象となる演算を施す
	int z_unit = ui->z_unit->itemData(ui->z_unit->currentIndex()).toInt();
	HT::Transform transform;
	TFMatrix<HT_COMPLEX_DATA_TYPE>& matrix_copy = transform.commonTFMatrix(*doc->getBaseUIData());
	TFMatrix<HT_COMPLEX_DATA_TYPE> matrix1;
	TFMatrix<HT_COMPLEX_DATA_TYPE> matrix2;

	switch (other_axis) {
	case GUIPlotParameterPanel::TF_AXIS::FREQ:
		matrix1 = matrix_copy.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::FRQ, other_axis_index1, other_axis_index1);;
		matrix2 = matrix_copy.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::FRQ, other_axis_index2, other_axis_index2);
		break;
	case GUIPlotParameterPanel::TF_AXIS::MIC:
		matrix1 = matrix_copy.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::MIC, other_axis_index1, other_axis_index1);;
		matrix2 = matrix_copy.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::MIC, other_axis_index2, other_axis_index2);
		break;
	case GUIPlotParameterPanel::TF_AXIS::SRCPOS:
		matrix1 = matrix_copy.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::SRC, other_axis_index1, other_axis_index1);;
		matrix2 = matrix_copy.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::SRC, other_axis_index2, other_axis_index2);
		break;
	default:
		THROW_CRITICAL("Unknown axis");
		break;
	}

	switch (z_unit) {
	case GNUPlot<>::ZUnit::AMPLITUDE:
		matrix1.abs();
		matrix2.abs();
	break;
	case GNUPlot<>::ZUnit::PHASE:
		matrix1.arg();
		matrix2.arg();
	break;
	case GNUPlot<>::ZUnit::DB:
		matrix1-=std::complex<double>(-1.0e-15, 0); // logゼロ防止
		matrix2-=std::complex<double>(-1.0e-15, 0);
		matrix1.db();
		matrix2.db();
	break;
	default:
		THROW_CRITICAL("Unknown Z axis's unit");
	}

	// それぞれの方法で差分を作成する
	int old_dim; // other_axisから旧データ型Matrixの次元番号を取得する
	switch (other_axis) {
	case TF_AXIS::MIC:
		old_dim = HT::MUSIC::UIData::MIC; // GSS でも同じ
		break;
	case TF_AXIS::SRCPOS:
		old_dim = HT::MUSIC::UIData::SRC;
		break;
	case TF_AXIS::FREQ:
		old_dim = HT::MUSIC::UIData::FRQ;
		break;
	default:
		THROW_CRITICAL("Unknown Axis");
		break;
	}

	// 差分
	switch (z_unit) {
	case GNUPlot<>::ZUnit::AMPLITUDE:
		// 割り算
		matrix1-=std::complex<double>(-1.0e-15, 0); // ゼロ割防止
		matrix2-=std::complex<double>(-1.0e-15, 0);
		matrix1 /= matrix2;
	break;
	case GNUPlot<>::ZUnit::PHASE:
		// 引き算
		matrix1 -= matrix2;
	break;
	case GNUPlot<>::ZUnit::DB:
		// 引き算
		matrix1 -= matrix2;		break;
	default:
		THROW_CRITICAL("Unknown Z axis's unit");
		break;
	}

	// グラフデータのセット
	matrix1.createBoostMatrix();
	graph.setMatrix(matrix1.boostMatrix(), indexes);

	// グラフのタイトル
	std::string title = createDiffTitle();
	graph.setTitle(title);

	// プロットスタイル
	int plot_style =
			ui->plot_style->itemData(ui->plot_style->currentIndex()).toInt();
	int surface = ui->surface->itemData(ui->surface->currentIndex()).toInt();
	int color_map =
			ui->color_map->itemData(ui->color_map->currentIndex()).toInt();

	graph.setStyle(plot_style, surface, color_map);

	// Z軸の単位 // 複素数マトリックスなので実数だけ取り出す
	graph.setZUnit(GNUPlot<>::ZUnit::REAL); // 差分の場合、既に単位が変換されているので、そのまま表示すればよい。

	// 軸のラベル
	std::vector<std::string> labels(4);
	labels[0] = "f[Hz]";
	labels[1] = "Mic[No.]";
	labels[2] = "Src[No.]";
	labels[3] = zUnitName(z_unit);
	graph.setLabels(labels);

	// 軸の重み
	std::vector<double> weights(4);
	weights[0] = double(fs) / window; // FFT
	weights[1] = 1; // SRC
	weights[2] = 1; // MIC
	weights[3] = 1;
	graph.setWeights(weights);

	// 軸の表示範囲
	std::string x_from = ui->x_from->text().toStdString();
	std::string x_to = ui->x_to->text().toStdString();
	std::string y_from = ui->y_from->text().toStdString();
	std::string y_to = ui->y_to->text().toStdString();
	std::string z_from = ui->z_from->text().toStdString();
	std::string z_to = ui->z_to->text().toStdString();
	std::pair<std::string, std::string> x_range(x_from, x_to);
	std::pair<std::string, std::string> y_range(y_from, y_to);
	std::pair<std::string, std::string> z_range(z_from, z_to);
	graph.setCurrentRange(x_range, y_range, z_range);

	return;
}

void GUIPlotParameterPanel::dispGraph() {

	try {

		int index = ui->display_target->currentIndex();
		int id = ui->display_target->itemData(index).toInt();

		switch (id) {
		case GUIPlotParameterPanel::TARGET::TF:
			{
				GNUPlot<std::complex<double>, 3> graph;
				presetTFGraph(graph);
				graph.plot(harktoolDir()); // グラフのプロット
			}
			break;
		case GUIPlotParameterPanel::TARGET::IFFT:
			{
				GNUPlot<std::complex<double>, 3> graph;
				presetIFFTGraph(graph);
				graph.plot(harktoolDir()); // グラフのプロット
			}
			break;
		case GUIPlotParameterPanel::TARGET::DIFF:
			{
				GNUPlot<std::complex<double>, 3> graph;
				presetDiffGraph(graph);
				graph.plot(harktoolDir()); // グラフのプロット
			}
			break;
		default:
			THROW_CRITICAL("Unknown GraphID");
			break;
		}

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIPlotParameterPanel::presetTFGraph(GNUPlot<std::complex<double>, 3>& graph) {

	// 軸の定義
	std::vector<int> indexes(3);
	int x_axis, y_axis, other_axis, other_axis_index;

	x_axis = ui->x_combo->itemData(ui->x_combo->currentIndex()).toInt();
	y_axis = ui->y_combo->itemData(ui->y_combo->currentIndex()).toInt();

	other_axis = otherUnit(GUIPlotParameterPanel::TARGET::TF, x_axis,y_axis);
	other_axis_index = ui->other_value_combo->itemData(
			ui->other_value_combo->currentIndex()).toInt();

	setIndex(&indexes, GNUPlot<>::AXIS::X, x_axis);
	setIndex(&indexes, GNUPlot<>::AXIS::Y, y_axis);
	setIndex(&indexes, other_axis_index, other_axis);

	// グラフオブジェクトの生成
	boost::multi_array<std::complex<double>, 3>* matrix;
	HT::Transform transform;
	TFMatrix<HT_COMPLEX_DATA_TYPE>& matrix_copy = transform.commonTFMatrix(*doc->getBaseUIData());
	matrix = matrix_copy.createBoostMatrix();
	graph.setMatrix(matrix, indexes);

	// グラフのタイトル
	std::string title = createTFTitle();
	graph.setTitle(title);

	// プロットスタイル
	int plot_style =
			ui->plot_style->itemData(ui->plot_style->currentIndex()).toInt();
	int surface = ui->surface->itemData(ui->surface->currentIndex()).toInt();
	int color_map =
			ui->color_map->itemData(ui->color_map->currentIndex()).toInt();
	int z_unit = ui->z_unit->itemData(ui->z_unit->currentIndex()).toInt();

	graph.setStyle(plot_style, surface, color_map);

	// Z軸の単位
	graph.setZUnit(z_unit);

	// 軸のラベル
	std::vector<std::string> labels(4);
	labels[0] = "f[Hz]";
	labels[1] = "Mic[No.]";
	labels[2] = "Src[No.]";
	labels[3] = zUnitName(z_unit);
	graph.setLabels(labels);

	// 軸の重み
	std::vector<double> weights(4);
	weights[0] = double(fs) / window; // FFT
	weights[1] = 1; // SRC
	weights[2] = 1; // MIC
	weights[3] = 1;
	graph.setWeights(weights);

	// 軸の表示範囲
	std::string x_from = ui->x_from->text().toStdString();
	std::string x_to = ui->x_to->text().toStdString();
	std::string y_from = ui->y_from->text().toStdString();
	std::string y_to = ui->y_to->text().toStdString();
	std::string z_from = ui->z_from->text().toStdString();
	std::string z_to = ui->z_to->text().toStdString();
	std::pair<std::string, std::string> x_range(x_from, x_to);
	std::pair<std::string, std::string> y_range(y_from, y_to);
	std::pair<std::string, std::string> z_range(z_from, z_to);
	graph.setCurrentRange(x_range, y_range, z_range);

	return;
}

void GUIPlotParameterPanel::presetIFFTGraph(GNUPlot<std::complex<double>, 3>& graph) {

	// 軸の定義
	std::vector<int> indexes(3);
	int x_axis, y_axis, other_axis, other_axis_index;

	x_axis = ui->x_combo->itemData(ui->x_combo->currentIndex()).toInt();
	y_axis = ui->y_combo->itemData(ui->y_combo->currentIndex()).toInt();
	other_axis = otherUnit(GUIPlotParameterPanel::TARGET::IFFT, x_axis, y_axis);
	other_axis_index = ui->other_value_combo->itemData(
			ui->other_value_combo->currentIndex()).toInt();
	setIndex(&indexes, GNUPlot<>::AXIS::X, x_axis);
	setIndex(&indexes, GNUPlot<>::AXIS::Y, y_axis);
	setIndex(&indexes, other_axis_index, other_axis);

	// グラフオブジェクトの生成
	HT::Transform transform;
	TFMatrix<HT_COMPLEX_DATA_TYPE>& ifft_matrix = transform.commonIFFTTFMatrix(*doc->getBaseUIData());
	ifft_matrix.createBoostMatrix();
	graph.setMatrix(ifft_matrix.boostMatrix(), indexes);

	// グラフのタイトル
	std::string title = createIFFTTitle();
	graph.setTitle(title);

	// プロットスタイル
	int plot_style =
			ui->plot_style->itemData(ui->plot_style->currentIndex()).toInt();
	int surface = ui->surface->itemData(ui->surface->currentIndex()).toInt();
	int color_map =
			ui->color_map->itemData(ui->color_map->currentIndex()).toInt();
	int z_unit = GNUPlot<>::ZUnit::REAL;

	graph.setStyle(plot_style, surface, color_map);

	// Z軸の単位
	graph.setZUnit(z_unit);

	// 軸のラベル
	std::vector<std::string> labels(4);
	labels[0] = "Time[Sec]";
	labels[1] = "Mic[No.]";
	labels[2] = "Src[No.]";
//	GNUPlot<>::ZUnit::AMPLITUDE
	labels[3] = zUnitName(GNUPlot<>::ZUnit::REAL);
//	labels[3] = zUnitName(z_unit);
	graph.setLabels(labels);

	// 軸の重み
	std::vector<double> weights(4);
	weights[0] = 1.0 / fs; // TIME
	weights[1] = 1; // SRC
	weights[2] = 1; // MIC
	weights[3] = 1;
	graph.setWeights(weights);

	// 軸の表示範囲
	std::string x_from = ui->x_from->text().toStdString();
	std::string x_to = ui->x_to->text().toStdString();
	std::string y_from = ui->y_from->text().toStdString();
	std::string y_to = ui->y_to->text().toStdString();
	std::string z_from = ui->z_from->text().toStdString();
	std::string z_to = ui->z_to->text().toStdString();
	std::pair<std::string, std::string> x_range(x_from, x_to);
	std::pair<std::string, std::string> y_range(y_from, y_to);
	std::pair<std::string, std::string> z_range(z_from, z_to);
	graph.setCurrentRange(x_range, y_range, z_range);

	return;
}

std::string GUIPlotParameterPanel::createTFTitle() {

	std::ostringstream os;

	int x_axis = ui->x_combo->itemData(ui->x_combo->currentIndex()).toInt();;
	int y_axis = ui->y_combo->itemData(ui->y_combo->currentIndex()).toInt();;
	int axis = otherUnit(GUIPlotParameterPanel::TARGET::TF, x_axis,y_axis);
	int index = ui->other_value_combo->itemData(ui->other_value_combo->currentIndex()).toInt();

	HT::MUSIC::UIDocument *music_doc=0;
	HT::GSS::UIDocument *gss_doc=0;
	if (dynamic_cast<HT::MUSIC::UIDocument *> (doc)) {
		music_doc = (HT::MUSIC::UIDocument*)doc;
	} else if (dynamic_cast<HT::GSS::UIDocument *> (doc)) {
		gss_doc = (HT::GSS::UIDocument*)doc;
	} else {
		THROW_CRITICAL("Unknown document");
	}

	switch(axis) {
	case GUIPlotParameterPanel::TF_AXIS::SRCPOS:
		os << " Transfer Function of Src. No. " << index;
		break;
	case GUIPlotParameterPanel::TF_AXIS::FREQ:
		os << " Transfer Function of " << index * fs / window << " [Hz]";
		break;
	case GUIPlotParameterPanel::TF_AXIS::MIC:
		os << " Transfer Function of Mic No. " << index;
		break;
	default:
		THROW_CRITICAL(QString("Unknown other parameter:%1").arg(axis));
	}

	if (music_doc) {
		os << "[" << HT::UIApp::getInstance()->GROUP_MUSIC << ">"
		<< music_doc->getName() << "]";
	} else if (gss_doc) {
		os << "[" << HT::UIApp::getInstance()->GROUP_GSS << ">"
		<< gss_doc->getName() << "]";
	}

	return os.str();
}

std::string GUIPlotParameterPanel::createDiffTitle() {

	std::ostringstream os;

	int x_axis = ui->x_combo->itemData(ui->x_combo->currentIndex()).toInt();;
	int y_axis = ui->y_combo->itemData(ui->y_combo->currentIndex()).toInt();;
	int axis = otherUnit(GUIPlotParameterPanel::TARGET::DIFF, x_axis,y_axis);
	int index = ui->other_value_combo->itemData(ui->other_value_combo->currentIndex()).toInt();
	int index2 = ui->other_value_combo2->itemData(ui->other_value_combo2->currentIndex()).toInt();

	HT::MUSIC::UIDocument *music_doc=0;
	HT::GSS::UIDocument *gss_doc=0;
	if (dynamic_cast<HT::MUSIC::UIDocument *> (doc)) {
		music_doc = (HT::MUSIC::UIDocument*)doc;
	} else if (dynamic_cast<HT::GSS::UIDocument *> (doc)) {
		gss_doc = (HT::GSS::UIDocument*)doc;
	} else {
		THROW_CRITICAL("Unknown document");
	}

	std::string diff_type;
	int z_unit = ui->z_unit->itemData(ui->z_unit->currentIndex()).toInt();
	HT::Transform transform;
	switch (z_unit) {
	case GNUPlot<>::ZUnit::AMPLITUDE:
		diff_type = " (division)";
	break;
	case GNUPlot<>::ZUnit::PHASE:
		diff_type = " (subtraction)";
		break;
	case GNUPlot<>::ZUnit::DB:
		diff_type = " (subtraction)";
		break;
	default:
		THROW_CRITICAL("Unknown z unit");
		break;
	}
	switch(axis) {
	case GUIPlotParameterPanel::TF_AXIS::SRCPOS:
		os << " Difference between No. " << index << " and " << index2 << " in Src." << diff_type;
		break;
	case GUIPlotParameterPanel::TF_AXIS::FREQ:
		os << " Difference between " << index * fs / window << " and " << index2 * fs / window << " [Hz] in Freq." << diff_type;
		break;
	case GUIPlotParameterPanel::TF_AXIS::MIC:
		os << " Difference between No." << index << " and " << index2 << " in Mic." << diff_type;
		break;
	default:
		THROW_CRITICAL("Unknown other parameter");
	}

	if (music_doc) {
		os << "[" << HT::UIApp::getInstance()->GROUP_MUSIC << ">"
		<< music_doc->getName() << "]";
	} else if (gss_doc) {
		os << "[" << HT::UIApp::getInstance()->GROUP_GSS << ">"
		<< gss_doc->getName() << "]";
	}
	return os.str();
}


std::string GUIPlotParameterPanel::createIFFTTitle() {
	std::ostringstream os;

	int x_axis = ui->x_combo->itemData(ui->x_combo->currentIndex()).toInt();;
	int y_axis = ui->y_combo->itemData(ui->y_combo->currentIndex()).toInt();;
	int axis = otherUnit(GUIPlotParameterPanel::TARGET::IFFT, x_axis,y_axis);
	int index = ui->other_value_combo->itemData(ui->other_value_combo->currentIndex()).toInt();

	HT::MUSIC::UIDocument *music_doc=0;
	HT::GSS::UIDocument *gss_doc=0;
	if (dynamic_cast<HT::MUSIC::UIDocument *> (doc)) {
		music_doc = (HT::MUSIC::UIDocument*)doc;
	} else	if (dynamic_cast<HT::GSS::UIDocument *> (doc)) {
		gss_doc = (HT::GSS::UIDocument*)doc;
	} else {
		THROW_CRITICAL("Unknown document");
	}

	switch(axis) {
	case GUIPlotParameterPanel::TF_AXIS::SRCPOS:
		os << " IFFT of Src No." << index;
		break;
	case GUIPlotParameterPanel::TF_AXIS::TIME:
		os << " IFFT of " << double(index) / fs << " [Sec]";
		break;
	case GUIPlotParameterPanel::TF_AXIS::MIC:
		os << " IFFT of Mic No." << index;
		break;
	default:
		THROW_CRITICAL("Unknown other parameter");
	}

	if (music_doc) {
		os << "[" << HT::UIApp::getInstance()->GROUP_MUSIC << ">"
				<< music_doc->getName() << "]";
	} else if (gss_doc) {
		os << "[" << HT::UIApp::getInstance()->GROUP_GSS << ">"
				<< gss_doc->getName() << "]";
	}
	return os.str();
}


// マトリックスの各次元は固定されているとみなして、GNUPlot用インデックスを作成する
void GUIPlotParameterPanel::setIndex(std::vector<int> *indexes, int xyz_axis, int specific_axis) {

	switch (specific_axis) {
	case GUIPlotParameterPanel::TF_AXIS::FREQ:
			(*indexes)[0] = xyz_axis;
		break;
	case GUIPlotParameterPanel::TF_AXIS::MIC:
		(*indexes)[1] = xyz_axis;
		break;
	case GUIPlotParameterPanel::TF_AXIS::SRCPOS:
		(*indexes)[2] = xyz_axis;
		break;
	case GUIPlotParameterPanel::TF_AXIS::TIME:
		(*indexes)[0] = xyz_axis;
		break;
	default:
		THROW_CRITICAL("Unknown axis");
		break;
	}
}
std::string GUIPlotParameterPanel::zUnitName(int id) {

	std::string label;
	switch(id) {
	case GNUPlot<>::ZUnit::AMPLITUDE:
	case GNUPlot<>::ZUnit::REAL:
		label = "Amplitude";
		break;
	case GNUPlot<>::ZUnit::PHASE:
		label = "Phase[deg.]";
		break;
	case GNUPlot<>::ZUnit::DB:
		label = "[dB]";
		break;
	default:
		THROW_CRITICAL("Unknow Z axis's unit");
		break;
	}
	return label;
}

void GUIPlotParameterPanel::setDisplayTargetToCombo(QComboBox *combo) {
	removeComboAll(combo);
//	combo->addItem("(Undefined)", QVariant(UNDEFINE));
	combo->addItem(tr("Transfer function", "伝達関数"), QVariant(GUIPlotParameterPanel::TARGET::TF));
	combo->addItem(tr("IFFT of Transfer function", "伝達関数のIFFT"), QVariant(GUIPlotParameterPanel::TARGET::IFFT));
	combo->addItem(tr("Difference of Transfer function", "伝達関数の差分"), QVariant(GUIPlotParameterPanel::TARGET::DIFF));
}

void GUIPlotParameterPanel::setTFAxisToCombo(QComboBox *combo) {

	removeComboAll(combo);
//	combo->addItem("(Undefined)", QVariant(UNDEFINE));
	combo->addItem(tr("Frequency", "周波数"), QVariant(GUIPlotParameterPanel::TF_AXIS::FREQ));
	combo->addItem(tr("Num of Sources", "音源番号"), QVariant(GUIPlotParameterPanel::TF_AXIS::SRCPOS));
	combo->addItem(tr("Num of Mics", "マイク番号"), QVariant(GUIPlotParameterPanel::TF_AXIS::MIC));
}

void GUIPlotParameterPanel::setIFFTAxisToCombo(QComboBox *combo) {

	removeComboAll(combo);
//	combo->addItem("(Undefined)", QVariant(GUIPlotParameterPanel::TF_AXIS::UNDEFINE));
	combo->addItem(tr("Time", "時間"), QVariant(GUIPlotParameterPanel::TF_AXIS::TIME));
	combo->addItem(tr("Num of Sources", "音源番号"), QVariant(GUIPlotParameterPanel::TF_AXIS::SRCPOS));
	combo->addItem(tr("Num of Mics", "マイク番号"), QVariant(GUIPlotParameterPanel::TF_AXIS::MIC));
}

void GUIPlotParameterPanel::setProtStyleCombo() {

	QComboBox *combo;
	combo = ui->plot_style;

	removeComboAll(combo);
	combo->addItem("(Undefined)", QVariant(GNUPlot<>::PlotStyle::UNDEFINE));
	combo->addItem("dots", QVariant(GNUPlot<>::PlotStyle::DOTS));
	combo->addItem("points", QVariant(GNUPlot<>::PlotStyle::POINTS));
	combo->addItem("lines", QVariant(GNUPlot<>::PlotStyle::LINES));
	combo->addItem("lines_points", QVariant(GNUPlot<>::PlotStyle::LINESPOINTS));

	combo = ui->surface;
	removeComboAll(combo);
	combo->addItem("(Undefined)", QVariant(GNUPlot<>::Surface::UNDEFINE));
	combo->addItem("line", QVariant(GNUPlot<>::Surface::LINE));
	combo->addItem("mesh", QVariant(GNUPlot<>::Surface::MESH));

	combo = ui->color_map;
	removeComboAll(combo);
	combo->addItem("(Undefined)", QVariant(GNUPlot<>::ColorMap::UNDEFINE));
	combo->addItem("pm3d", QVariant(GNUPlot<>::ColorMap::PM3D));
	combo->addItem("pm3d map", QVariant(GNUPlot<>::ColorMap::PM3D_MAP));
	combo->addItem("pm3d at b", QVariant(GNUPlot<>::ColorMap::PM3D_AT_B));

	combo = ui->z_unit;
	removeComboAll(combo);
	combo->addItem("Amplitude", QVariant(GNUPlot<>::ZUnit::AMPLITUDE));
	combo->addItem("dB", QVariant(GNUPlot<>::ZUnit::DB));
	combo->addItem("Phase", QVariant(GNUPlot<>::ZUnit::PHASE));

}

void GUIPlotParameterPanel::reset() {

	try {
		QObject *obj = sender();

		if (obj == ui->reset_param) {
			// プロットスタイル
			selectCombo(ui->plot_style, GNUPlot<>::PlotStyle::LINES);
			selectCombo(ui->surface, GNUPlot<>::Surface::LINE);
			selectCombo(ui->color_map, GNUPlot<>::ColorMap::UNDEFINE);
			ui->plot_style->setDisabled(false);
			ui->surface->setDisabled(false);
			ui->color_map->setDisabled(false);

		} else if (obj == ui->reset) {
			selectCombo(ui->display_target, GUIPlotParameterPanel::TARGET::TF);

			// 表示範囲
			ui->x_from->setText("");
			ui->x_to->setText("");
			ui->y_from->setText("");
			ui->y_to->setText("");
			ui->z_from->setText("");
			ui->z_to->setText("");
			// Z軸単位
			ui->z_unit->setCurrentIndex(0);

		} else {
			THROW_CRITICAL("Unknown reset target");
		}

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIPlotParameterPanel::exportData() {

	try {
		QString fname = QFileDialog::getSaveFileName(this, HT::UIDocument::getLastPath().c_str());
		if (fname == "") {
			return;
		}
		HT::UIDocument::setLastPath(fname.toStdString());

		QFile file(fname);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
			THROW_WARNING(QString(tr("Can't open file:%1")).arg(fname));
			return;
		}

		GNUPlot<std::complex<double>, 3> graph;


		int index = ui->display_target->currentIndex();
		int id = ui->display_target->itemData(index).toInt();

		switch (id) {
		case GUIPlotParameterPanel::TARGET::TF:
			presetTFGraph(graph);
			graph.exportCSVToFile(fname.toStdString());

			break;
		case GUIPlotParameterPanel::TARGET::IFFT:
			presetIFFTGraph(graph);
			graph.exportCSVToFile(fname.toStdString(),false);
			break;
		default:
			THROW_CRITICAL(QString("Unknown target id:%1").arg(id));
			connectPlotCombo(true);
			break;
		}

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

GUIPlotParameterPanel::~GUIPlotParameterPanel() {
	delete ui;
}

void GUIPlotParameterPanel::changeEvent(QEvent *e) {
	QDialog::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
