#include "GUITSPListFileTemplateNew.h"
#include "ui_GUITSPListFileTemplateNew.h"

#include <math.h>
#include <QtGui>
#include <QDir>
#include <QDateTime>
#include "GUICommon.h"
#include "UIDocument.h"


GUITSPListFileTemplateNew::GUITSPListFileTemplateNew(QWidget *parent) :
	QDialog(parent), ui(new Ui::GUITSPListFileTemplateNew) {
	ui->setupUi(this);

	::setWindowTitle(this);

	QDir dir;
	QString path;

	// TSP リストファイル
#ifdef DEBUG
	path = TEST_TSP_TEMPLATE;
#else
	path = QString("%1/YOUR_PATH/tsp_D#DEGREE#.wav").arg(dir.homePath());
#endif

	// TSPリストファイル
	ui->file_list_template->setText(path);

	// オリジナルインパルス
	QString app_dir = QCoreApplication::applicationDirPath();
	path = QString("%1/%2/%3").arg(app_dir).arg(HARK_TOOLS_DIR.c_str()).arg(IMPULSE_ORG.c_str());
	ui->original_impulse->setText(path);
}

GUITSPListFileTemplateNew::~GUITSPListFileTemplateNew() {
	delete ui;
}

void GUITSPListFileTemplateNew::fileListTemplate() {

	try {

		QString fname = QFileDialog::getOpenFileName(this, "", HT::UIDocument::getLastPath(ui->file_list_template->text().toStdString()).c_str());
		if (fname != "") {
			ui->file_list_template->setText(fname);
			HT::UIDocument::setLastPath(fname.toStdString());
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUITSPListFileTemplateNew::originalImpulseFile() {

	try {
		QString fname = QFileDialog::getOpenFileName(this, "", HT::UIDocument::getLastPath(ui->original_impulse->text().toStdString()).c_str());
		if (fname != "") {
			ui->original_impulse->setText(fname);
			HT::UIDocument::setLastPath(fname.toStdString());
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUITSPListFileTemplateNew::createTemplate() {

	try {

		QStringList lines;

		double from = ui->phi_from->value();
		double to = ui->phi_to->value();
		from = (from >= to ? from - 360: from);
		double interval = ui->interval->value();
		double radius = ui->radius->value();
		double theta = ui->theta->value();
		QString list_file_template =  ui->file_list_template->text();
		int synchronous_avg = ui->sync_avg->value();
		int signal_max = ui->signal_max->value();
		int tsp_offset = ui->tsp_offset->value();
		QString original_impulse = ui->original_impulse->text();
		int offset = ui->impulse_offset->value();
		QString channel_list = ui->channel_list->text();
		QString use_channel_list = (ui->use_channel_list->isChecked()? "" : "#");

		double count = double(to - from) / interval;
		if (count == (to - from) / interval) {
			count++;
		}
		lines << QString("# TSP-ListFile generated from harktool by %1.").arg(getUserName());
		lines << QString("# DateTime: %1").arg(QDateTime::currentDateTime().toString("yyyy/MM/dd HH:mm:ss"));
		lines << "TSP-ListFile";
		lines << "Version 1.1";
		lines << QString("SynchronousAverage = %1 #  Num of Synchronous averaging.").arg(synchronous_avg);
		lines << QString("OriginalImpulseFltFile = %1").arg(original_impulse);
		lines << QString("TSPOffset = %1 # Starting with 1").arg(tsp_offset);
		lines << QString("SignalMax = %1").arg(signal_max);
		lines << QString("%1ChannelList = %2 # Wav channel No. for use").arg(use_channel_list).arg(channel_list);

		lines << "";
		lines << "# List of Sound source position and TSP file.";
		lines << "# x, y, z %% TSP file name, IR start with potision in sample(starting with 1).";
		for (double phi = from, i = 0; phi <= to; phi += interval, i++) { // 小数のループなので注意
			double x = radius * cos(theta * M_PI / 180) * cos(phi * M_PI / 180);
			double y = radius * cos(theta * M_PI / 180) * sin(phi * M_PI / 180);
			double z = radius * sin(theta * M_PI / 180);

			QString filename( list_file_template );
			//QString angle = QString("%1").arg((phi < 0? phi + 360: phi), 3+1+2, 'f', 2, QLatin1Char('0')); // "045.05"の様な文字列の作成
			QString angle = QString("%1").arg((phi < 0? phi + 360: phi), 3, 'f', 0, QLatin1Char('0')); // "045"の様な文字列の作成
			angle.replace(".", "_");
			filename.replace("#DEGREE#", QString("%1").arg(angle));
			lines << QString("%1, %2, %3 \%\%%4, %5").arg(x,7,'f',4).arg(y,7,'f',4).arg(z,7,'f',4).arg(filename).arg(offset);
		}
		// 最終行にはノイズの音源の情報を付加する
		lines << "# The following line is used for TF genaration for separation only when FIXEX_MODE=true.";
		lines << QString("#%1, %2, %3 \%\%%4, %5").arg(0.0,7,'f',4).arg(0.0,7,'f',4).arg(-1.0,7,'f',4).arg("TSP_FOR_NOISE").arg(offset);

		QString text = lines.join("\n");
		text += "\n";
		emit createdTemplate(text);
		close();

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUITSPListFileTemplateNew::changeEvent(QEvent *e) {
	QDialog::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
