#ifndef UIAPP_H
#define UIAPP_H

#include <iostream>
#include <typeinfo>

#include "UITFDocument.h"
#include "UITFData.h"
#include "UIDatDocument.h"
#include "common.h"

namespace HT {

namespace MUSIC {
	class UIDocument;
	class UIData;
}

namespace GSS {
	class UIDocument;
	class UIData;
}

namespace MIC {
	class UIDocument;
}

//! UIアプリケーションクラス
/*!
 * シングルトン生成のトップレベルクラス
 */
class UIApp {
private:
public:
	static UIApp *app; //! アプリケーションインスタンスポインタ
	//! ドキュメントのグループID
	typedef enum {
		 UNDEFINEDID=-1
		,GSS
		,MUSIC
		,MIC
		,GSSMAT
		,NOISE
		,TSP
		,IMPULSE
	} GROUPID;
	//! ドキュメントグループ名
	//\{
	static const char* GROUP_GSS;
	static const char* GROUP_MUSIC;
	static const char* GROUP_MIC;
	static const char* GROUP_GSSMAT;
	static const char* GROUP_NOISE;
	static const char* GROUP_TSP;
	static const char* GROUP_IMPULSE;
	//\}

	UIApp() {}
	virtual ~UIApp();
private:

public:
	//! アプリケーションインスタンスの取得
	static UIApp *getInstance() {

		if (!app) {
			app = new UIApp;
		}
		return app;
	}

	// UIDocumentの型からグループIDを取得する
	template <typename T> HT::UIApp::GROUPID groupID() {
		const std::type_info& info = typeid(T);

		if (info == typeid(HT::MUSIC::UIDocument)) {
			return GSS;
		} else if (info == typeid(HT::MUSIC::UIDocument)) {
			return MUSIC;
		} else if (info == typeid(HT::MIC::UIDocument)) {
			return MIC;
//		} else if (info == typeid(HT::GSSMAT::UIDocument)) {
//			return GSSMAT;
		} else if (info == typeid(HT::NOISE::UIDocument)) {
			return NOISE;
		} else if (info == typeid(HT::TSP::UIDocument)) {
			return TSP;
		} else if (info == typeid(HT::IMPULSE::UIDocument)) {
			return IMPULSE;
		}
		return (HT::UIApp::GROUPID)-1;
	}
	UIApp::GROUPID groupID(HT::UIDocument* doc);

	const char* groupName(HT::UIApp::GROUPID id) {

		switch(id) {
		case GSS:
			return GROUP_GSS;
			break;
		case MUSIC:
			return GROUP_MUSIC;
			break;
		case MIC:
			return GROUP_MIC;
			break;
		case GSSMAT:
			return GROUP_GSSMAT;
			break;
		case NOISE:
			return GROUP_NOISE;
			break;
		case TSP:
			return GROUP_TSP;
			break;
		case IMPULSE:
			return GROUP_IMPULSE;
			break;
		default:
			THROW_CRITICAL("Unknown GroupID");
			break;
		}
	}
	// harkioのIDをharktool用に変換する
	HT::UIApp::GROUPID convIDHarkToolTextToDocument(int id);
};

// グローバルデータを１度だけ初期化
__declspec(selectany) const char* UIApp::GROUP_GSS     = "Separation-TF";
__declspec(selectany) const char* UIApp::GROUP_GSSMAT  = "Separation-Matrix";
__declspec(selectany) const char* UIApp::GROUP_MUSIC   = "Localization-TF";
__declspec(selectany) const char* UIApp::GROUP_MIC     = "MIARY-LocationFile";
__declspec(selectany) const char* UIApp::GROUP_NOISE   = "Noise-LocationFile";
__declspec(selectany) const char* UIApp::GROUP_TSP     = "TSP-ListFile";
__declspec(selectany) const char* UIApp::GROUP_IMPULSE = "ImpulseResponse-ListFile";

__declspec(selectany) UIApp* UIApp::app = NULL;

} //namespace
#endif
