


//#define GPL 1

#include <complex>
#include <cmath>
#include <fstream>
#include "UITFData.h"
#include "UIDatData.h"
#include "UIGraph.h"
#include "UIDatDocument.h"
#include "Matrix.h"
#include "libharkio/libharkio.h"
#include "libharkio/textdata.h"
#include "common.h"
#include <QApplication>
#include <QCoreApplication>
#include <boost/format.hpp>
#include <boost/regex.hpp>
#include <boost/shared_ptr.hpp>
#include "../libgui/GUICommon.h"
 


#ifdef GPL
//	#include <fftw.h>
	#include <fftw3.h>

#elif defined(HARKLICENSE)
	#include "fft.hpp"
#else
	#define HARKLICENSE
	#include "fft.hpp"
#endif

using namespace std;
using namespace harkio;
using namespace boost;

namespace HT {
namespace MUSIC {

UIData::UIData(UIDocument* document) {
	HT::UIData::document = document;
	tsp_canceled=false;
}
UIData::~UIData() 
{
IFDEBUG(DP("# MUSIC::UIData Destructor"));
}

HT::MUSIC::UIDocument* UIData::getDocument() 
{
cerr << "UIData::getDocument start" << endl;

	return dynamic_cast<HT::MUSIC::UIDocument*>(document);
cerr << "UIData::getDocument end" << endl;
}

TFMatrix<HT_COMPLEX_DATA_TYPE>* UIData::IFFT(TFMatrix<HT_COMPLEX_DATA_TYPE>& matrix, int FFT, int NMic, int NSrc) {

	// 結果を戻す配列
	TFMatrix<HT_COMPLEX_DATA_TYPE> *result_matrix = new TFMatrix<HT_COMPLEX_DATA_TYPE>(DM3, FFT, NMic, NSrc);
#ifdef GPL
IFDEBUG(DP("# in GPL"));



	TFMatrix<HT_COMPLEX_DATA_TYPE> copy_matrix(DM3, FFT, NMic, NSrc); // 周波数ラインを元に戻す配列
	// matrix -> result へ(半分 + 1)コピー
	copy_matrix.range(DM3, FRQ, 0, FFT/2) = matrix.range(DM3, FRQ, 0, FFT/2);

	// 残り半分へ、複素共役をとってコピー
	for (int i=1; i < FFT/2; i++) { // 1から開始
//		matrix.range(DM3, FRQ, i, i).conj();
		copy_matrix.range(DM3, FRQ, FFT-i, FFT-i) = matrix.range(DM3, FRQ, i, i);
		copy_matrix.range(DM3, FRQ, FFT-i, FFT-i).conj();
	}

	// 逆FFT
	fftw_complex *in, *out;
	fftw_plan p;

	in  = (fftw_complex *)fftw_malloc(sizeof(fftw_complex) * FFT);
	out = (fftw_complex *)fftw_malloc(sizeof(fftw_complex) * FFT);
	p   = fftw_create_plan(FFT, FFTW_BACKWARD, FFTW_ESTIMATE); 	// 逆FFT

	for(int mic =0; mic < NMic; mic++) {
		for(int src =0; src < NSrc; src++) {
			// fftw 領域へコピー
			for(int fft =0; fft < FFT; fft++) {
				HT_COMPLEX_DATA_TYPE *mat = &(copy_matrix.matrix[fft][mic][src]);
				in[fft].re = mat->real();
				in[fft].im = mat->imag();
			}
			// Execute
			fftw_one(p, in, out);
			// 結果の取得
			for(int fft =0; fft < FFT; fft++) { // 同じループ
				result_matrix->matrix[fft][mic][src] = HT_COMPLEX_DATA_TYPE(out[fft].re, out[fft].im);
			}
		}
	}
	fftw_destroy_plan(p);
	fftw_free(in);
	fftw_free(out);


IFDEBUG(DP("# out GPL"));

#endif

#ifdef HARKLICENSE

IFDEBUG(DP("# in HARKLICENCE"));

	HRI::rfft<HT_DATA_TYPE> fft_obj(FFT);
	for (int mic = 0; mic < NMic; mic++) {
		for (int src = 0; src < NSrc; src++) {
			for (int fft = 0; fft < FFT/2.0+1; fft++) { // fftwとの違いに注意
				fft_obj.A[fft] = matrix.matrix[fft][mic][src];
			}
//			fft_obj.analyze();
			fft_obj.synthesize();
			for (int fft = 0; fft < FFT; fft++) {
				result_matrix->matrix[fft][mic][src] = HT_COMPLEX_DATA_TYPE(fft_obj.a[fft], 0);
			}
		}
	}
#endif
	return result_matrix;
}

// TFファイルの計算して作成する
int UIData::createTFData(const CreateTFParam& param) {
IFDEBUG(DP("# createTFData"));

	// パラメータのコピー
	int FFT = param.NFFT;
	int Fs = param.NFS;
	int MUSIC_LEN = param.NFFT;
	int MUSIC_DIR = param.DIRECT_LENGTH;
	int NIR = param.IMPULSE_RESPONSE_LEN;
	int NSrc;
	int NMic;

	vector<harkio::UIMic<> > mic_list;
	vector<harkio::UISrc<> > src_list;
	TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_H;
	QString arg_channel_list;

	try {
//		int const MAX_SIZE = 100;
		int MAX_SIZE; //対象区間の最大値
		MAX_SIZE = 200;

		enum FILEMODE {
			UNDEFINE, TSP, IMPULSE
		};
		FILEMODE file_mode = UNDEFINE; // 何も指定がなければ(古い仕様の場合)、インパルス応答とみなす。

		harkio::TextData files;
		harkio::TextBase *file;

		// マイクファイル読み込み
		file = files.readFile(param.mic_file);
		mic_list = ((harkio::MICARYLocationFile*)file)->mic_lists;
//cout << "--------mic_lists" << param.mic_file << endl;

		// ソースファイル読み込み
		int ret;
		ret = files.checkFileIdentifier(param.src_file);

		switch (ret) {
		case harkio::TextData::TYPE::TSPListFile:
			file_mode = TSP;
			file = files.readFile(param.src_file);
			src_list = ((harkio::TSPListFile*)file)->src_list;
			((harkio::TSPListFile*) file)->atLeastVersion(harkio::TSPListFile::V1_0);
			break;
		case harkio::TextData::TYPE::ImpulseResponseListFile:
			file_mode = IMPULSE;
			file = files.readFile(param.src_file);
			src_list = ((harkio::ImpulseResponseListFile*)file)->src_list;
			((harkio::ImpulseResponseListFile*) file)->atLeastVersion(harkio::ImpulseResponseListFile::V1_0);
			break;
		default:
			QString msg = QString("Unkown format file:%1").arg(ret);
			THROW_CRITICAL(msg);
			break;
		}
		std::auto_ptr<harkio::TextBase> tempptr(file);

		NSrc = src_list.size();
		NMic = mic_list.size();

		TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_music(3, MUSIC_LEN, NMic, NSrc); // for MUSIC
		std::string temp_dir;

		try {
			if (file_mode == TSP) {
				// 作業用テンポラリディレクトリの作成
				char temp[PATH_MAX];
				snprintf(temp, sizeof(temp) - 1, "%s/%d-XXXXXX", harktoolDir().c_str(), getpid());
				temp_dir = mkdtemp(temp);

				// エラーチェック、channel_listの作成
				vector<int> channel_list  = ((harkio::TSPListFile*)file)->channel_list;
				if (channel_list.size() && mic_list.size() != channel_list.size()) {
					QString msg = QString("TSP-FileList's Num of ChannelList:%1 is different from MICARY-LoacationFile: %2").arg(channel_list.size()).arg(mic_list.size());
					THROW_WARNING(msg);
				}
				for (uint i = 0; i < channel_list.size(); i++) {
					if (i) {
						arg_channel_list += ",";
					}
					arg_channel_list += QString("%1").arg(channel_list[i]);
				}

			}
			// TFMatrix生成初期化
			for (int src = 0; src < NSrc; src++) {
				string implement_file = src_list[src].implement_file;
				int find_pos = -1;
				if (file_mode != TSP) {
					find_pos = implement_file.find("*", 0);
					if (find_pos < 0) {
						QString msg = QString("Can't find '*' character to replace the channel No. :%1").arg(implement_file.c_str());
						THROW_WARNING(msg);
					}
				}
				string impulse_base_for_tsp;
				for (int mic = 0; mic < NMic; mic++) {
					string impulseresponse = implement_file;
					if (file_mode == TSP) {

						// 途中経過をイベントループに通知
						emit createdTSP(src, NSrc, this);
						QApplication::processEvents();
						if (tsp_canceled) {
							THROW_WARNING("TSP generation canceled.");
						}
						// マイク番号が変化する毎に、チャンネル数分のFLTファイルを作成する
						if (mic == 0) {
							impulse_base_for_tsp = fullPathToFile(src_list[src].implement_file);
							QString app_dir = QCoreApplication::applicationDirPath();
							string utils_dir = app_dir.toStdString() + "/" + HARK_TOOLS_DIR;
							string original_impulse_arg;
							TSPListFile *tspfile = ((TSPListFile*) file);
							if (checkSoundFile(tspfile->originalImpulse())) {
								original_impulse_arg = " --tsporigwav ";
							} else {
								original_impulse_arg = " --tsporigflt ";
							}
							string command = utils_dir + "/" + TSP_TO_IMPULSE /* 変換コマンド */
							+ original_impulse_arg + tspfile->originalImpulse() /* オリジナルインパルス */
							+ " --sigmax " + lexical_cast<string> (tspfile->signalMax()) /* シグナルマックス */
							+ " --syncaddfreq " + lexical_cast<string> (tspfile->syncAvg()) /* 同期加算回数 */
							+ " --syncaddoffset " + lexical_cast<string> (tspfile->tspOffset()) /* TSP オフセット */
							+ " " + implement_file /* 入力音声ファイル */
							+ " " + temp_dir + "/" + impulse_base_for_tsp; /* 出力インパルス応答ファイルのベース名 */
							/* チャンネルリスト */
							if (arg_channel_list != "") {
								command +=  " --outchannellist  " + arg_channel_list.toStdString();
							}

							 cout << command << endl;
							int ret = system(command.c_str());
							if (WEXITSTATUS(ret)) {
								QString msg = QString("Command Error:\n%1").arg(command.c_str());
								THROW_CRITICAL(msg);
							}
						}
						
						impulseresponse = (boost::format("%s/%s_ch%d.flt") %temp_dir %impulse_base_for_tsp  %(mic + 1)).str();


					} 
					else if (file_mode == IMPULSE) 
					{
						impulseresponse.replace(find_pos, 1, boost::lexical_cast<string>(mic + 1));
					}

					// FLT Open
					ifstream ifs(impulseresponse.c_str(), ios::binary);
					if (ifs.fail()) 
					{
						cerr << "Can't open flt file:" << impulseresponse.c_str() << endl;
						QString msg = QString("Can't open flt file:%1").arg(impulseresponse.c_str());
						THROW_CRITICAL(msg);
					}


//					cerr << "open flt file:" << impulseresponse.c_str() << endl;

					// cutstart 分ＦＬＴをシーク
					ifs.seekg(((src_list[src].cut_start) - 1) * sizeof(HT_FLT_DATA_TYPE), std::ios::beg);

					// TFMatrixに fltファイルの中身を格納
					HT_FLT_DATA_TYPE buf;
					int fft = 0;
					while (!ifs.eof()) {
						ifs.read(reinterpret_cast<char*> (&buf),
								sizeof(HT_FLT_DATA_TYPE));
						matrix_music.matrix[fft][mic][src] = buf;
						fft++;
						if (fft == MUSIC_LEN) { // FOR MUSIC
							break;
						}
					}
					if (fft != MUSIC_LEN) {
						QString msg = QString("Can't read all of flt data. Read size: %1").arg(fft);
						THROW_CRITICAL(msg);
					}
				}
			}
			if (file_mode == TSP) {
				// 終了通知
				emit createdTSP(-1, NSrc, this);
				// テンポラリ削除
				std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
				int ret = system(temp.c_str());
				(void)ret;
			}

		} catch (...) {
			// 終了通知
			emit createdTSP(-1, NSrc, this);
			throw;
		}

		// for MUSICTF calculation
		// src_listとmatrix_musicの削除するインデックスをerase_listに保存する(matlabではこの様になっている)
		// (後から分かった仕様だが、一番最後のデータのみでよい)
		vector<int> erase_list;
		for (unsigned int i = 0; i < src_list.size(); i++) {
			// (0,0,-1) の座標を削除する
			if (src_list[i].x == 0 && src_list[i].y == 0 && src_list[i].z == -1) {
				erase_list.push_back(i); // MATLABのソースとは違い、消す部分を探索している
			}
		}
		// src_listとmatrix_musicの削除処理
		// ここで、SrcPosが更新される
		for (unsigned int erase_offset = 0; erase_offset < erase_list.size(); erase_offset++) {
			// erase_listの内容が昇順になっている事が必要
			int new_offset = erase_list[erase_offset] - erase_offset; //削除すると順番がずれる
			// src_listから削除
			vector<harkio::UISrc<> >::iterator it = src_list.begin();
			it += new_offset;
			src_list.erase(it);
			// matrix_musicから削除
			matrix_music.cut_trim(3, SRC, new_offset, new_offset);
		}
		// サイズが変わったので NSrcの更新
		NSrc = src_list.size();


		/*--------------------------------------------------------------------------------------------------*/
		/*         matrix_musicに0埋め                                                                         */
		/*--------------------------------------------------------------------------------------------------*/
		int max_index = 0;
		for (int src = 0; src < NSrc; src++) 
		{
			for (int mic = 0; mic < NMic; mic++) 
			{
				TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_music_temp;
				matrix_music_temp = matrix_music.range(6, MIC, mic, mic, SRC,src, src);
				matrix_music_temp.abs().complex_max(&max_index);
				int N = max_index + MUSIC_DIR + 1; // complex_max のmax_indexは0から始まる

				if( src_list[src].cut_end > 0 )
				{
					N = src_list[src].cut_end;
				}

				matrix_music.range(9, FRQ, N, MUSIC_LEN - 1, MIC, mic, mic,SRC, src, src) = 0;
			}
		}



		/*--------------------------------------------------------------------------------------------------*/
		/*         ファイル名表示                                                                                          */
		/*--------------------------------------------------------------------------------------------------*/
		max_index = 0;
		int find_pos=-1;
		int dspflg=0;
		int blankflg=0;
		string impulseresponse;
		string impulse_base_for_tsp;	
		

		


		/*--------------------------------------------------------------------------------------------------*/
		/*        ファイル名表示                                                                                           */
		/*--------------------------------------------------------------------------------------------------*/
		for (int src = 0; src < NSrc; src++) 
		{
			blankflg=1;

			for (int mic = 0; mic < NMic; mic++) 
			{
				TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_music_temp;
				matrix_music_temp = matrix_music.range(6, MIC, mic, mic, SRC,src, src);
				matrix_music_temp.abs().complex_max(&max_index);
//				int N = max_index + MUSIC_DIR + 1; // complex_max のmax_indexは0から始まる

				//反響音が大きい場合
				if ( max_index >  MAX_SIZE ) 
				{

					/*--------------------------------------------------------------------------------------------------*/
					/*        ヘッダー表示                                                                                              */
					/*--------------------------------------------------------------------------------------------------*/
					if(dspflg == 0)
					{
						cout << "\n" << endl;
						cout << "     ----------------------------------------------------------------------------" << endl;
						cout << "            直接音より反射音が大きい可能性のあるファイル名       " << endl;
						cout << "     ----------------------------------------------------------------------------" << endl;
					}


					// ファイル名をコンソールに表示する //
					switch(file_mode) 
					{
					case IMPULSE:
						find_pos=-1;
						impulseresponse = src_list[src].implement_file ;
						find_pos = impulseresponse.find("*", 0);
						impulseresponse.replace( find_pos, 1, boost::lexical_cast<string>(mic+1));
						impulseresponse = impulseresponse.substr( impulseresponse.find_last_of("/")+1); //ファイル名のみ取得
						break;
					default:
						impulse_base_for_tsp = fullPathToFile(src_list[src].implement_file);
						impulseresponse = (boost::format("  %s  ch%d")  %impulse_base_for_tsp  %(mic + 1)).str();
						break;

					}//switch end

					if(blankflg ==1)
					{
						cout << "" << endl;
						blankflg=0;
					}

					cout << impulseresponse.c_str() << " [ 最大ピーク値: " << max_index << " ]"  << endl; 	

					dspflg++;
				
				}//if
				
			}//for end
		}//for end


		if(dspflg>0)
		{
			
			QMessageBox::information(0, tr("information"), 
                                        tr(" 直接音より反射音が大きい可能性のあるファイル名を、\nコンソールに表示しました。確認してください。"));
		}

		cout << "" << endl;
		cout << "" << endl;


		TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_H(DM3, FFT, NMic, NSrc); // 結果出力用

		IFDEBUG(DP("EXEC FFT"));
#ifdef GPL
		// FFT
		IFDEBUG(DP("************************ FFT  GPL **********"));


		fftw_complex *in, *out;
		fftw_plan p;

		in = (fftw_complex *)fftw_malloc(sizeof(fftw_complex) * FFT);
		out = (fftw_complex *)fftw_malloc(sizeof(fftw_complex) * FFT);
		p = fftw_create_plan(FFT, FFTW_FORWARD, FFTW_ESTIMATE);

		for(int mic =0; mic < NMic; mic++) 
		{
			for(int src =0; src < NSrc; src++) 
			{
				// fftw 領域へコピー
				for(int fft =0; fft < FFT; fft++) 
				{
					HT_COMPLEX_DATA_TYPE *mat = &(matrix_music.matrix[fft][mic][src]);
					in[fft].re = mat->real();
					in[fft].im = 0;
				}
				// Execute
				fftw_one(p, in, out);

				// 結果の取得
				// 同じループ				
				for(int fft =0; fft < FFT; fft++) 
				{ 
					matrix_H.matrix[fft][mic][src] = HT_COMPLEX_DATA_TYPE(out[fft].re, out[fft].im);
				}
			}
		}
		fftw_destroy_plan(p);
		fftw_free(in);
		fftw_free(out);

#endif


#ifdef HARKLICENSE
		IFDEBUG(DP("fftw use HARKLICENSE"));

		HRI::fft<float> fft_obj(FFT);
		for (int mic = 0; mic < NMic; mic++) {
			for (int src = 0; src < NSrc; src++) {
				for (int fft = 0; fft < FFT; fft++) {
					fft_obj.a[fft] = matrix_music.matrix[fft][mic][src];
				}
				fft_obj.analyze(); // fft実行
				for (int fft = 0; fft < FFT; fft++) {
					matrix_H.matrix[fft][mic][src] = fft_obj.A[fft];
				}
			}
		}
#endif
		IFDEBUG(DP("FFT end"));

		// matrix_H を半分にする
		matrix_H.trim(3, FRQ, 0, FFT / 2);
		//	matrix_H.range(3, 0, FFT/2, FFT/2) = 0; // MUSICでは、この処理はしていない。

		// 新規作成時に、データをhakrioのコンテナにすべて移動する
		moveToHARKIOParam param;
		param.NMic = NMic;
		param.NFFT = FFT;
		param.NIR = NIR;
		param.NFS = Fs;

		vector<HT::UIMic<HT_DATA_TYPE> > mic_list2;
		vector<HT::UISrc<HT_DATA_TYPE> > src_list2;
		src_list2.resize(src_list.size());
		mic_list2.resize(mic_list.size());

		for (uint i=0; i < src_list.size(); i++) 
		{
			src_list2[i].convertFromHarkIO(src_list[i]);
		}
		for (uint i=0; i < mic_list.size(); i++) {
			mic_list2[i].convertFromHarkIO(mic_list[i]);
		}
		param.src_list = &src_list2;
		param.mic_list = &mic_list2;
		param.tf_matrix = &matrix_H;

		moveToHARKIO(param);
	} catch (...) {
		throw;
	}
	return 0;
}

harkio::HARK* UIData::moveToHARKIO(const moveToHARKIOParam& param) {

	vector<HT::UISrc<HT_DATA_TYPE> >& src_list = *param.src_list;
	vector<HT::UIMic<HT_DATA_TYPE> >& mic_list = *param.mic_list;
	TFMatrix<HT_COMPLEX_DATA_TYPE>& tf_matrix = *param.tf_matrix;

	int NMic = param.NMic;
	int NFFT = param.NFFT;
	int NIR  = param.NIR;
	int NFS  = param.NFS;

	// ライブラリのデータコンテナに、データをコピーする
	harkio::HARK*hark = new HARK;
	setHARKIO(hark);
	try {
		// Data, Chunkのポインタを用意しておく
		Data *hgtf;

		// HGTF Data 生成
		hgtf = new HGTF();
		hark->addData(hgtf);

		// FMT Chunk　生成
		FMT *fmt  = new FMT(FMT::V1_0, FMT::M2PG, NMic, NFFT, NIR, NFS);
		hgtf->addChunk(fmt);
		FMT::MIC_LIST_TYPE *mic = fmt->newMic();
		// ここでのコピーは、mic_listの内容によっては不完全である。
		for(unsigned i=0; i < mic_list.size(); i++) {
			(*mic)[i].x = mic_list[i].x;
			(*mic)[i].y = mic_list[i].y;
			(*mic)[i].z = mic_list[i].z;
		}
		// M2PG Chunk 生成
		M2PG *m2pg = new M2PG(M2PG::V1_0, fmt, src_list.size(), 1); // NDは、データ格納時に、src_list.size()として、-1を除いたサイズに更新されている
		hgtf->addChunk(m2pg);

		// M2PG Chunk に音源位置座標コンテナを生成して、src_list()からデータをセット
		M2PG::SRC_LIST_TYPE *src = m2pg->getSrc();

		// ここでのコピーは、src_listの内容によっては不完全である。
		for (unsigned int srcn=0; srcn < src_list.size(); srcn++) {
			(*src)[srcn][0].x = src_list[srcn].x;
			(*src)[srcn][0].y = src_list[srcn].y;
			(*src)[srcn][0].z = src_list[srcn].z;
//DP((*src)[srcn][0].x, (*src)[srcn][0].y, (*src)[srcn][0].z);
		}

		// M2PG Chunk に伝達関数コンテナを生成して、getMatrix()からデータをセット
		M2PG::TF_LIST_TYPE *tf = m2pg->getTF();
		for(unsigned int src=0; src < src_list.size();src++) { // ライブラリTFデータをコピーする
			for(unsigned int mic=0; mic < mic_list.size(); mic++) {
				for(int fft=0; fft < NFFT/2+1; fft++) {
					HT_COMPLEX_DATA_TYPE value = tf_matrix.matrix[fft][mic][src];
					(*tf)[src][0][mic][fft].real() = (HT_MUSIC_DATA_TYPE)value.real();
					(*tf)[src][0][mic][fft].imag() = (HT_MUSIC_DATA_TYPE)value.imag();
				}
			}
		}
		// SPEC Chunkの設定(ここでは内容は設定しない)
		SPEC *spec = new SPEC(SPEC::V1_0);
		hgtf->addChunk(spec);
		return hark;

	} catch(std::exception& expt) {
		setHARKIO(NULL);
		harkio::Error* err = dynamic_cast<harkio::Error*>(&expt);
		if (err) {
			THROW_CRITICAL(err->getMsg().c_str());
		}
		throw;
	}
}


int UIData::saveTFFile(std::string fullpath_file) {
IFDEBUG(DP("# saveTFFile start"));

	try {
		// ファイルへ保存
		harkio::HARK *hark = this->getHARKIO();
		if(!hark) {
			THROW_CRITICAL("Data is not harkio's data");
		}
		hark->writeFile(fullpath_file.c_str());

	} catch(harkio::Error &e) {
		THROW_CRITICAL(e.getMsg().c_str());
	}
IFDEBUG(DP("saveTFFile end"));

	return 0;
}


} //namespace MUSIC
} //namespace
