#ifndef UIDATDOCUMENT_H
#define UIDATDOCUMENT_H

#include <iostream>
#include <string>
#include <sstream>
#include <map>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <utility>

#include "UIDatData.h"
#include "UITFDocument.h"
#include "UIDocument.h"
#include "UIGraph.h"
#include "common.h"

namespace HT {
class UIGraph;
class UIData;

namespace MUSIC {
class UIData;

// メンバ関数用パラメータ
class MKMusicParam {
public:
	std::string mic_file;
	int NFFT;
	int NFS;
	int DEGREE_INTERVAL;
	int IMPULSE_RESPONSE_LEN;
};


//! MUSIC TF用のドキュメントクラス
class UIDocument : public HT::UIDocument {
private:
	std::string srcin_filename;	// srcinfファイル名
	std::string mic_filename;	// micファイル名
	boost::shared_ptr<HT::MUSIC::UIData> data;

public:
	UIDocument(std::string name, std::string filename);
	UIDocument();
	virtual ~UIDocument();

	void setSrcInfFile(std::string name) {
		srcin_filename = name;
	}
	std::string& getSrcInfFile() {
		return srcin_filename;
	}

	void setMicFile(std::string name) {
		mic_filename = name;
	}
	std::string& getMicFile() {
		return mic_filename;
	}

	//! このドキュメントに対して、Dataインスタンスの生成と包含を同時に行う
	virtual UIData* newTFData(UIDocument *doc);

	UIData* getUIData(bool exception=true);
	HT::UIData* getBaseUIData();

	void mkmusictf(MKMusicParam& param);
	//! ドキュメントに、ファイルを読み込む
	void loadFile(std::string filename);
	//!  harkio でファイルを読み込む
	void loadFileByLib(std::string filename);
	//! 　harkioを使わないでファイルを読み込む
	void loadFileNormally(std::string filename);
public:

};

} //namespace MUSIC
} //namespace
#endif
