#ifndef UIDOCUMENT_H
#define UIDOCUMENT_H

#include <iostream>
#include <string>
#include <sstream>
#include <map>
#include <boost/lexical_cast.hpp>
#include <vector>
#include <utility>
#include <sys/stat.h>

#include "UIData.h"
#include "common.h"

namespace HT {
class UIApp;
class UIData;

//! 全ドキュメントの抽象基底クラス
class UIDocument : public QObject {
private:
public:
	//! 全てのDocumentに対して、最後にR/WしたPATHを保持
	/*!
	 * ファイルオープンウィンドウを開く時に
	 * 一番最後にアクセスしたディレクトリを開く為に使用している。
	 */
	UIDocument() {
		name = "Untitled";
	}
	virtual ~UIDocument() {}
	virtual HT::UIData* getBaseUIData();

	static std::string last_path;
	//! 最後にアクセスしたパスを設定
	static void setLastPath(std::string path) {
		last_path = fullPathToPath(path);
	}
	//! 最後にアクセスしたパスを取得
	static std::string& getLastPath() {
		return last_path;
	}
	//! 引数に指定された、ファイルが開けなければ普通のgetLastPathを取得する
	static std::string getLastPath(std::string file) {
		struct stat st;
		stat(file.c_str(), &st);
		if (S_ISDIR(st.st_mode) || S_ISREG(st.st_mode)) {
			return file;
		}
		return getLastPath();
	}
	//! ドキュメントの名前を取得する
	std::string& getName() {
		return name;
	}
	//! ドキュメントのファイル名をフルパスで取得する
	std::string& getFileName() {
		return filename;
	}

	//! ドキュメント名をセットする(重複しない)
	void setName(std::string name) {
		this->name = name;
	}
	//! ドキュメントのフルパスファイル名を設定する
	void setFileName(std::string name) {
		filename = name;
		if (name != "") { // filenameは１度も保存されていない場合は空にしているので
			setName(fullPathToFile(name));
		}
	}
protected:
	std::string name;
	std::string filename;
	HT::UIApp* app;

};

class UITextData{
private:
	std::string _text;
public:
	std::string& text() {
		return _text;
	}
};

namespace MIC {
class UIDocument : public HT::UIDocument {
private:
public:
	UIDocument(std::string name, std::string filename) {
		setName(name);
		setFileName(filename);
	}
	UIDocument(){}
	~UIDocument() {}
	UITextData text;
};
} // MIC

namespace TSP {
class UIDocument : public HT::UIDocument {
private:
public:
	UIDocument(std::string name, std::string filename) {
		setName(name);
		setFileName(filename);
	}
	UIDocument(){}
	~UIDocument() {}
	UITextData text;

};
} // TSP

/*
namespace GSSMAT {
class UIDocument : public HT::UIDocument {
private:
public:
	UIDocument(std::string name, std::string filename) {
		setName(name);
		setFileName(filename);
	}
	UIDocument(){}
	~UIDocument() {}
	UITextData text;
};
} // GSSMAT
*/
namespace IMPULSE {
class UIDocument : public HT::UIDocument {
private:
public:
	UIDocument(std::string name, std::string filename) {
		setName(name);
		setFileName(filename);
	}
	UIDocument(){}
	~UIDocument() {}
	UITextData text;
};
} // IMPULSE

namespace NOISE {
class UIDocument : public HT::UIDocument {
private:
public:
	UIDocument(std::string name, std::string filename) {
		setName(name);
		setFileName(filename);
	}
	UIDocument(){}
	~UIDocument() {}
	UITextData text;
};
} // NOISE


__declspec(selectany) std::string UIDocument::last_path = "";
} //namespace
#endif
