#include "UIVerify.h"
#include "UIApp.h"
#include "UITFDocument.h"
#include "common.h"

using namespace std;
namespace HT {

void UIVerify::verifyGSS() {
IFDEBUG(DP("#verifyGSS"));
	total_count=1;
	tab1_error=0;
	tab2_error=0;

	HT::UIApp *app = HT::UIApp::getInstance();
	// ドキュメント
	HT::GSS::UIDocument* doc1 = app->getDocument<HT::GSS::UIDocument*>(tab1);
	HT::GSS::UIDocument* doc2 = app->getDocument<HT::GSS::UIDocument*>(tab2);
	// データ
	HT::GSS::UIData *data1 = doc1->getUIData();
	HT::GSS::UIData *data2 = doc2->getUIData();
	// マトリックス取得
	TFMatrix<HT_COMPLEX_DATA_TYPE> *matrix1 = data1->getMatrix();
	TFMatrix<HT_COMPLEX_DATA_TYPE> *matrix2 = data2->getMatrix();

	//双方のサイズチェック
	if (matrix1->size().size() == matrix2->size().size()) {
		for (unsigned int i=0; i < matrix1->size().size(); i++) {
			if (matrix1->size()[i] != matrix2->size()[i]) {
				THROW_WARNING("Can't execute verify because data size is different between tabs.");
			}
			total_count *= matrix1->size()[i];
		}
	}
	total_count*=2;	// 複素数なので2倍する
	
	vector<int> size1 = matrix1->size();
	vector<double> diff_temp;
	vector<double> diff_ratio_temp;
	double diff;		// 差分
	double diff_ratio; // 差分率
	double ratio_target;	// 割合の分母

	for(int i = 0; i < size1[0]; i++) {
		for(int j = 0; j < size1[1]; j++) {
			for(int k = 0; k < size1[2]; k++) {
				// データチェック
				if (!isfinite(matrix1->matrix[i][j][k].real())) {
					tab1_error++;
				} 
				if (!isfinite(matrix1->matrix[i][j][k].imag())) {
					tab1_error++;
				}
				if (!isfinite(matrix2->matrix[i][j][k].real())) {
					tab2_error++;
				}
				if (!isfinite(matrix2->matrix[i][j][k].imag())) {
					tab2_error++;
				}
				// 差分
				// 実部
				diff = abs(matrix1->matrix[i][j][k].real() - matrix2->matrix[i][j][k].real());
				ratio_target = abs(matrix2->matrix[i][j][k].real());
				diff_ratio = diff/ ratio_target;
				diff_temp.push_back(diff);
				if (ratio_target) {
					diff_ratio_temp.push_back(diff_ratio);
				}
				// 虚部
				diff = abs(matrix1->matrix[i][j][k].imag() - matrix2->matrix[i][j][k].imag());
				ratio_target = abs(matrix1->matrix[i][j][k].imag() + matrix2->matrix[i][j][k].imag())/2;
				diff_temp.push_back(diff);
				if (ratio_target) {
					diff_ratio = diff/ ratio_target;
					diff_ratio_temp.push_back(diff_ratio);
				}
			}
		}
	}
	// 差分の統計情報計算
	IFDEBUG(DP("total_count", total_count, tab1_error, tab2_error));
	diff_info = new CheckVector<double>(diff_temp);
	diff_info->execCheck();
	IFDEBUG(diff_info->print());

	diff_ratio_info = new CheckVector<double>(diff_ratio_temp);
	diff_ratio_info->execCheck();
	IFDEBUG(diff_ratio_info->print());
}
void UIVerify::verifyMUSIC() {
IFDEBUG(DP("#verifyMUSIC"));
	total_count=1;
	tab1_error=0;
	tab2_error=0;
	// ドキュメント
	HT::UIApp *app = HT::UIApp::getInstance();
	HT::MUSIC::UIDocument* doc1 = app->getDatDocument(tab1);
	HT::MUSIC::UIDocument* doc2 = app->getDatDocument(tab2);
	// データ
	HT::MUSIC::UIData *data1 = doc1->getUIData();
	HT::MUSIC::UIData *data2 = doc2->getUIData();
	// マトリックス取得
	TFMatrix<HT_COMPLEX_DATA_TYPE> *matrix1 = data1->getMatrix();
	TFMatrix<HT_COMPLEX_DATA_TYPE> *matrix2 = data2->getMatrix();

	//双方のサイズチェック
	if (matrix1->size().size() == matrix2->size().size()) {
		for (unsigned int i=0; i < matrix1->size().size(); i++) {
			if (matrix1->size()[i] != matrix2->size()[i]) {
				THROW_WARNING("Can't execute verify because data size is different between tabs.");
			}
			total_count *= matrix1->size()[i];
		}
	}
	total_count*=2;	// 複素数なので2倍する
			
	vector<int> size1 = matrix1->size();
	vector<double> diff_temp;
	vector<double> diff_ratio_temp;
	double diff;		// 差分
	double diff_ratio; // 差分率
	double ratio_target;	// 割合の分母

	for(int i = 0; i < size1[0]; i++) {
		for(int j = 0; j < size1[1]; j++) {
			for(int k = 0; k < size1[2]; k++) {
				// データチェック
				if (!isfinite(matrix1->matrix[i][j][k].real())) {
					tab1_error++;
				} 
				if (!isfinite(matrix1->matrix[i][j][k].imag())) {
					tab1_error++;
				}
				if (!isfinite(matrix2->matrix[i][j][k].real())) {
					tab2_error++;
				}
				if (!isfinite(matrix2->matrix[i][j][k].imag())) {
					tab2_error++;
				}
				// 差分
				// 実部
				diff = abs(matrix1->matrix[i][j][k].real() - matrix2->matrix[i][j][k].real());
				ratio_target = abs(matrix2->matrix[i][j][k].real());
				diff_temp.push_back(diff);
				if (ratio_target) {
					diff_ratio = diff/ ratio_target;
					diff_ratio_temp.push_back(diff_ratio);
				}
				diff = abs(matrix1->matrix[i][j][k].imag() - matrix2->matrix[i][j][k].imag());
				ratio_target = abs(matrix1->matrix[i][j][k].imag() + matrix2->matrix[i][j][k].imag())/2;
				diff_temp.push_back(diff);
				if (ratio_target) {
					diff_ratio = diff/ ratio_target;
					diff_ratio_temp.push_back(diff_ratio);
				}
			}
		}
	}
	// 差分の統計情報計算
	IFDEBUG(DP("total_count", total_count, tab1_error, tab2_error));
	diff_info = new CheckVector<double>(diff_temp);
	diff_info->execCheck();
	IFDEBUG(diff_info.print());

	diff_ratio_info = new CheckVector<double>(diff_ratio_temp);
	diff_ratio_info->execCheck();
	IFDEBUG(diff_ratio_info.print());
}

void UIVerify::createGraphDiff(int mic, HT::UIVerify::TARGET target, HT::ZAxis::ZAXIS axis) {

	FILE *fp;
	ostringstream tmp_os;

	// 比較対象のマトリックス
	TFMatrix<HT_COMPLEX_DATA_TYPE> *tab1_matrix;
	TFMatrix<HT_COMPLEX_DATA_TYPE> *tab2_matrix;
	string zaxis_title = "";
	
	// タイトル準備
	string title;
	string main_title;
	switch(target) {
	case (UIVerify::DIFF):
		tmp_os << "Difference ";
		main_title = tmp_os.str();
		tmp_os << "between " << tab1 << " and " << tab2 
			<< "at MicNo.=" << boost::lexical_cast<string>(mic);
		break;
	case (UIVerify::DIFF_RATIO):
		tmp_os << "Difference % "; 
		main_title = tmp_os.str();
		tmp_os << "of " << tab1 << " to " << tab2 
			<< "at MicNo.=" << boost::lexical_cast<string>(mic);
		break;
	}
	title = tmp_os.str();
	tmp_os.str("");

	
	// 軸タイトル準備
	string axis_title;
	switch(axis) {
	case(HT::ZAxis::AMPLITUDE):
		tmp_os << main_title << "of Amplitude ";
		break;
	case(HT::ZAxis::DB):
		tmp_os << main_title << "of deciBel ";
		break;
	case(HT::ZAxis::PHASE):
		tmp_os << main_title << "of Phase ";
		break;
	default:
		THROW_EXCEPTION("");
	}
	axis_title = tmp_os.str();
	tmp_os.str("");
	
	// GSS,MUSIC を判別して、マトリックスのデータを取得する
	switch(group) {
	case(HT::UIApp::GSS):
	{
		HT::GSS::UIData* data;
		data = dynamic_cast<HT::GSS::UIData*>(tab1_data);
		tab1_matrix = data->getMatrix();
		data = dynamic_cast<HT::GSS::UIData*>(tab2_data);
		tab2_matrix = data->getMatrix();
		break;
	}
	case(HT::UIApp::MUSIC):
	{
		HT::MUSIC::UIData* data;
		data = dynamic_cast<HT::MUSIC::UIData*>(tab1_data);
		tab1_matrix = data->getMatrix();
		data = dynamic_cast<HT::MUSIC::UIData*>(tab2_data);
		tab2_matrix = data->getMatrix();
		break;
	}
	default:
		THROW_EXCEPTION("");
		break;
	}

	// テンポラリファイルオープン
	char filename[128] = "./temp-XXXXXX";
	int fd =  mkstemp(filename);
	fp = fdopen(fd, "w");
	if (!fp) {
		THROW_WARNING(string("Can't open tempfile :") + filename);
	}
	HT::UIGraph *graph = new UIGraph();
	graph->temp_file = string(filename);
	graph_list.push_back(graph);	// グラフの保持
	
	// 表題
//	tmp_os << "Difference between " << tab1 << " and " << tab2 << "at MicNo.=" << boost::lexical_cast<string>(mic);
	fprintf(fp, "set title '%s'\n", title.c_str());
//	tmp_os.str("");

	// ラベル
	fprintf(fp,"set xlabel 'f[Hz]'\n");
	fprintf(fp,"set ylabel 'Src[No.]'\n");
	fprintf(fp,"set zlabel '%s'\n", axis_title.c_str());

	// XYZ軸
	fprintf(fp,"set xzeroaxis lt -1\n");
	fprintf(fp,"set yzeroaxis lt -1\n");
	fprintf(fp,"set zeroaxis lt -1\n");
	fprintf(fp,"set ticslevel 0\n"); // Z軸交点

	// PlotStyle
	fprintf(fp, "splot '-' notitle %s\n", "with lines");

	// GSS
	if(group == HT::UIApp::GSS) {
		HT::GSS::UIData* data;
		data = dynamic_cast<HT::GSS::UIData*>(tab1_data);

		// tab1,tab2共に、FFT,FS,SRCのサイズが同じである事はチェック済み
		// 対象データコピー
		TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_copy1 = tab1_matrix->range(HT::GSS::UIData::DM3, HT::GSS::UIData::MIC, mic, mic);;	
		TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_copy2 = tab2_matrix->range(HT::GSS::UIData::DM3, HT::GSS::UIData::MIC, mic, mic);	

		// Z軸の値の計算
		{
				switch(axis) {
				case(HT::ZAxis::AMPLITUDE):
				{
					matrix_copy1.abs();
					matrix_copy2.abs();
					break;
				}
				case(HT::ZAxis::DB):
				{
					matrix_copy1.db();
					matrix_copy2.db();
					break;
				}
				case(HT::ZAxis::PHASE):
				{
					matrix_copy1.arg();
					matrix_copy2.arg();
					break;
				}
				default:
					THROW_EXCEPTION("");
					break;
			}
		}
		// 差分計算
		matrix_copy1.range(GSS::UIData::DM3, GSS::UIData::MIC, mic, mic)-=matrix_copy2.range(GSS::UIData::DM3, GSS::UIData::MIC, mic, mic);
		
		matrix_copy1.abs();
		matrix_copy2.abs();
		if (target == UIVerify::DIFF_RATIO) {
			matrix_copy1.range(GSS::UIData::DM3, GSS::UIData::MIC, mic, mic)/=matrix_copy2.range(GSS::UIData::DM3, GSS::UIData::MIC, mic, mic);
			matrix_copy1.range(GSS::UIData::DM3, GSS::UIData::MIC, mic, mic)*=100;
		}
		for(int nSrc=0; nSrc< data->getNSrc(); nSrc++) {
			for(int nFreq = 0; nFreq < data->getNFreq(); nFreq++) {
				HT_DATA_TYPE diff = (HT_DATA_TYPE)matrix_copy1.matrix[nFreq][mic][nSrc].real(); // 絶対値を取っているのでこれでよい
				int freqLine = nFreq;
				tmp_os
				<< freqLine * data->getNFS() / data->getNFFT()
				<< " "
				<< nSrc
				<< " "
				<< diff;
				fprintf(fp, "%s\n", tmp_os.str().c_str());
				tmp_os.str("");
			}
			fprintf(fp, "%s\n", "");
		}
	// MUSIC
	} else if(group == HT::UIApp::MUSIC) {
		HT::MUSIC::UIData* data;
		data = dynamic_cast<HT::MUSIC::UIData*>(tab1_data);

		// tab1,tab2共に、FFT,FS,SRCのサイズが同じである事はチェック済み
		// 対象データコピー
		TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_copy1 = tab1_matrix->range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::MIC, mic, mic);;	
		TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_copy2 = tab2_matrix->range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::MIC, mic, mic);	
		// Z軸の値の計算
		{
				switch(axis) {
				case(HT::ZAxis::AMPLITUDE):
				{
					matrix_copy1.abs();
					matrix_copy2.abs();
					break;
				}
				case(HT::ZAxis::DB):
				{
					matrix_copy1.db();
					matrix_copy2.db();
					break;
				}
				case(HT::ZAxis::PHASE):
				{
					matrix_copy1.arg();
					matrix_copy2.arg();
					break;
				}
				default:
					THROW_EXCEPTION("");
					break;
			}
		}
		// 差分割合計算
		matrix_copy1.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::MIC, mic, mic)-=matrix_copy2.range(HT::MUSIC::UIData::DM3, HT::MUSIC::UIData::MIC, mic, mic);
		matrix_copy1.abs();
		matrix_copy2.abs();
		if (target == UIVerify::DIFF_RATIO) {
			matrix_copy1.range(MUSIC::UIData::DM3, MUSIC::UIData::MIC, mic, mic)/=matrix_copy2.range(MUSIC::UIData::DM3, MUSIC::UIData::MIC, mic, mic);
			matrix_copy1.range(MUSIC::UIData::DM3, MUSIC::UIData::MIC, mic, mic)*=100;
		}
		for(int nSrc=0; nSrc< data->getNSrc(); nSrc++) {
			for(int nFreq = 0; nFreq < data->getNFreq(); nFreq++) {
				HT_DATA_TYPE diff = (HT_DATA_TYPE)matrix_copy1.matrix[nFreq][mic][nSrc].real(); // 絶対値を取っているのでこれでよい
				int freqLine = nFreq;
				tmp_os
				<< freqLine * data->getNFS() / data->getNFFT()
				<< " "
				<< nSrc
				<< " "
				<< diff;
				fprintf(fp, "%s\n", tmp_os.str().c_str());
				tmp_os.str("");
			}
			fprintf(fp, "%s\n", "");
		}
	}
	fprintf(fp, "e\n");
	fprintf(fp, "pause -1\n");
	fclose(fp);

	// プロットの実行
	graph->plot();

}


} //namespace
